unit CsvText;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.Menus, Vcl.StdCtrls,
  Vcl.ComCtrls, IniFiles, mPlugin, System.UITypes;

const
  FORM_WIDTH_SET       = 770;
  FORM_HEIGHT_SET      = 500;
  FORM_MAX             = 'MAX';
  FORM_MIN             = 'MIN';
  FORM_NORMAL          = 'NORMAL';

  FORM_LEFT_SHIFT      = 9;
  FORM_TOP_SHIFT       = 9;

  INIFILE_NAME         = 'MeryCsvText.ini';
  DLL_FOLDER_NAME      = 'Plugins';
  CONVERT_FOLDER_NAME  = 'MeryCsvText';
  CONVERT_FILE_NAME    = 'CsvFormat.txt';
  TAB_FILE_NAME        = 'TabSeparateValue.txt';
  UNKOWN_DST_LIST      = 'UnKnown.txt';

  PARENT_LINE_NUMBER   = 'LineNumber';
  PARENT_ITEM_LF       = 'ItemLf';
  PARENT_SHOW_MERY     = 'ShowMery';

  CSVLIST_ITEM_MAX     = 2048;

  CHAR_UNKNOW_CHECK_SIZE_MAX = 4095;
  ENCODE_AUTO             = 0;
  ENCODE_SJIS             = 1;
  ENCODE_JIS              = 2;
  ENCODE_EUCJP            = 3;
  ENCODE_UTF7             = 4;
  ENCODE_UTF8_BOM_ON      = 5;
  ENCODE_UTF8_BOM_OFF     = 6;
  ENCODE_UTF16_BOM_ON_BE  = 7;
  ENCODE_UTF16_BOM_OFF_BE = 8;
  ENCODE_UTF16_BOM_ON_LE  = 9;
  ENCODE_UTF16_BOM_OFF_LE = 10;
  ENCODE_UNKNOWN          = 20;
  CHAR_AUTO               = 'o';
  CHAR_SJIS               = 'Shift_JIS';
  CHAR_JIS                = 'JIS';
  CHAR_EUCJP              = 'EUC-JP';
  CHAR_UTF7               = 'UTF7';
  CHAR_UTF8_BOM_ON        = 'UTF8  [BOML]';
  CHAR_UTF8_BOM_OFF       = 'UTF8  [BOM]';
  CHAR_UTF16_BOM_ON_BE    = 'UTF16 [BOML BigEndian]';
  CHAR_UTF16_BOM_OFF_BE   = 'UTF16 [BOM BigEndian]';
  CHAR_UTF16_BOM_ON_LE    = 'UTF16 [BOML LittleEndian]';
  CHAR_UTF16_BOM_OFF_LE   = 'UTF16 [BOM LittleEndian]';
  CHAR_UNKNOWN            = 's';
  CHAR_BINARY             = 'oCit@C';
  RET_CRLF                = 'Cr+Lf(Windows) ';
  RET_CR                  = 'Cr(OS9)';
  RET_LF                  = 'Lf(OSX,Android)';
  RET_NONE                = '';
  RET_UNKNOWN             = 's';
  RETURN_CRLF             = 0;
  RETURN_CR               = 1;
  RETURN_LF               = 2;
  RETURN_NONE             = 3;
  EDITOR_READ_ONLY        = '';
  FILE_READ_ONLY          = 't@Cǎp';

type
  SJISString   = type AnsiString(932);   // SJIS
  EUCString    = type AnsiString(20932); // EUC
  JISString    = type AnsiString(50220); // JIS ISO-2022JP
  UTF7String   = type AnsiString(65000); // UTF7
  UTF8String   = type AnsiString(65001); // UTF8

//20932	EUC-JP	Japanese (JIS 0208-1990 and 0212-1990)	{ (JIS 0208-1990 and 0212-1990)
//51932	euc-jp	Japanese (EUC)	{ (EUC) ̓_ł
//50220	iso-2022-jp	Japanese (JIS)	{ (JIS)
//65000	utf-7	Unicode (UTF-7)	Unicode (UTF-7)
//65001	utf-8	Unicode (UTF-8)	Unicode (UTF-8)

type
  TFormCsvText = class(TForm)
    StatusBar1: TStatusBar;
    Panel1: TPanel;
    Memo9: TMemo;
    Memo1: TMemo;
    RadioGroup1: TRadioGroup;
    ButtonShowMery: TButton;
    ButtonSave: TButton;
    ButtonCancel: TButton;
    MainMenu1: TMainMenu;
    MenuFile: TMenuItem;
    MenuClose: TMenuItem;
    MenuCode: TMenuItem;
    MenuReadSJIS: TMenuItem;
    MenuReadJIS: TMenuItem;
    MenuReadEUCJP: TMenuItem;
    MenuReadUTF7: TMenuItem;
    MenuReadUTF8Bom: TMenuItem;
    MenuReadUTF8: TMenuItem;
    MenuReadUTF16BomBe: TMenuItem;
    MenuReadUTF16Be: TMenuItem;
    MenuReadUTF16BomLe: TMenuItem;
    MenuReadUTF16Le: TMenuItem;
    MenuReadAuto: TMenuItem;
    MenuView: TMenuItem;
    MenuShowLineNumber: TMenuItem;
    MenuSeparator41: TMenuItem;
    MenuTab2: TMenuItem;
    MenuTab4: TMenuItem;
    MenuTab6: TMenuItem;
    MenuTab8: TMenuItem;
    MenuSetting: TMenuItem;
    MenuFontColorReadOnly: TMenuItem;
    MenuBackColorReadOnly: TMenuItem;
    MenuShowMerySetting: TMenuItem;
    MenuSeparator43: TMenuItem;
    MenuSeparator42: TMenuItem;
    MenuLineSetSetting: TMenuItem;
    MenuLfSetting: TMenuItem;
    SaveDialog1: TSaveDialog;
    FontDialog1: TFontDialog;
    ColorDialog1: TColorDialog;
    Timer6: TTimer;
    Timer1: TTimer;

    procedure WmGetMinMaxInfo(var msg : TWMGetMinMaxInfo); message WM_GETMINMAXINFO;

    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);

    procedure ButtonCancelClick(Sender: TObject);
    procedure ButtonShowMeryClick(Sender: TObject);
    procedure ButtonSaveClick(Sender: TObject);

    procedure RadioGroup1Click(Sender: TObject);

    procedure Timer1Timer(Sender: TObject);
    procedure Timer6Timer(Sender: TObject);

    procedure MenuCloseClick(Sender: TObject);
    procedure MenuShowLineNumberClick(Sender: TObject);
    procedure MenuTab2Click(Sender: TObject);
    procedure MenuTab4Click(Sender: TObject);
    procedure MenuTab6Click(Sender: TObject);
    procedure MenuTab8Click(Sender: TObject);
    procedure MenuFontColorReadOnlyClick(Sender: TObject);
    procedure MenuBackColorReadOnlyClick(Sender: TObject);
    procedure MenuReadSJISClick(Sender: TObject);
    procedure MenuReadJISClick(Sender: TObject);
    procedure MenuReadEUCJPClick(Sender: TObject);
    procedure MenuReadUTF7Click(Sender: TObject);
    procedure MenuReadUTF8BomClick(Sender: TObject);
    procedure MenuReadUTF8Click(Sender: TObject);
    procedure MenuReadUTF16BomBeClick(Sender: TObject);
    procedure MenuReadUTF16BeClick(Sender: TObject);
    procedure MenuReadUTF16BomLeClick(Sender: TObject);
    procedure MenuReadUTF16LeClick(Sender: TObject);
    procedure MenuReadAutoClick(Sender: TObject);
    procedure MenuLineSetSettingClick(Sender: TObject);
    procedure MenuLfSettingClick(Sender: TObject);
    procedure MenuShowMerySettingClick(Sender: TObject);

    function GetcurrentFileName(hwnd: THandle): String;

    procedure SetLineNumber(Sender: TObject);
    procedure MemoStatus(Sender: TObject);

    procedure StringToCsvListEx1(SrcString: String; CsvList: TstringList);
    procedure StringToCsvList(SrcString: String; CsvList: TstringList);

    procedure LoadFileAuto(Sender: TObject);
    procedure LoadFile(Sender: TObject);
    procedure LoadTabSeparateFile(Sender: TObject);
    procedure LoadCsvFile(Sender: TObject);
    procedure SetCsvFile(Sender: TObject);

    procedure GetCharCodeRetCode(FileName:String; var CharCode: Integer; var RetCode: Integer; Sender: TObject);

    procedure Memo1KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Memo9Click(Sender: TObject);

  private
    { Private 錾 }
    FormPosLeft: Integer;
    FormPosTop: Integer;
    FormPosWidth: Integer;
    FormPosHeight: Integer;
    FormPosRight: Integer;
    FormPosButtom: Integer;

    MonitorTop: Integer;
    MonitorLeft: Integer;
    MonitorRight: Integer;
    MonitorButtom: Integer;
    MonitorWidth: Integer;
    MonitorHeight: Integer;

    StrFormState: String;

    IntFormShiftX: Integer;
    IntFormShiftY: Integer;

    IntTopLineMemo: Integer;
    IntLoadFileSize: Integer;
    IntReadEncode1: Integer;
    IntReadReturn1: Integer;

    StrLoadChar: String;
    StrLoadReturn: String;
    StrSaveChar: String;
    StrSaveReturn: String;
    StrSaveFolderFileCsv: String;
    StrLoadCharMem: String;
    StrLoadReturnMem: String;
    StrSaveCharMem: String;
    StrSaveReturnMem: String;

    BoolLfInLine: Boolean;
    BoolUnKnownCharCode: Boolean;
    BoolFileReadOnly: Boolean;
    BoolTypeText: Boolean;
    BoolTypeCsvAuto: Boolean;
    BoolTypeCsvAutoFixed: Boolean;
    BoolTypeCsvComma: Boolean;
    BoolTypeCsvCommaFixed: Boolean;
    BoolFormShow: Boolean;

    StrListLoadCsvFile: TStringList;
    StrListLoadFileTmp: TStringList;
    StrListTextEx: TstringList;

    TabStops: array [0..0] of DWord;
    CsvMax: array [0..CSVLIST_ITEM_MAX] of Integer;

    StrCsvSeparater: String;
    StrReturnMark: String;
    StrLoadFileName: String;
    IntEditorHwnd: HWND;

  public
    { Public 錾 }

  end;

var
  FormCsvText: TFormCsvText;

implementation

{$R *.dfm}

uses Setting;

procedure TFormCsvText.WmGetMinMaxInfo(var msg : TWMGetMinMaxInfo);
begin
  //ŏTCY
  with msg.MinMaxInfo^.ptMinTrackSize do
  begin
    x := FORM_WIDTH_SET;
    y := FORM_HEIGHT_SET;
  end;
  //őTCY
  with msg.MinMaxInfo^.ptMaxTrackSize do
  begin
    //
  end;
  //ő剻TCY
  with msg.MinMaxInfo^.ptMaxSize do
  begin
    //
  end;
end;

procedure TFormCsvText.FormClose(Sender: TObject; var Action: TCloseAction);
begin
;
end;

procedure TFormCsvText.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
;
end;

procedure TFormCsvText.FormCreate(Sender: TObject);
var
  sStr1: String;
  IniFile: TIniFile;
begin
  //
  IntFormShiftX  := 0;
  IntFormShiftY  := 0;

  //
  Timer1.Enabled := FALSE;
  Timer6.Enabled := FALSE;

  //
  BoolFormShow   := FALSE;

  //m
  StrListLoadCsvFile := TStringList.Create;
  StrListLoadFileTmp := TStringList.Create;
  StrListTextEx := TStringList.Create;

  //tH_쐬
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + CONVERT_FOLDER_NAME;
  if DirectoryExists(sStr1) = FALSE then
  begin
    ForceDirectories(sStr1);
  end;

  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //tH[ʒu
  FormPosLeft   := IniFile.ReadInteger('FormCsvText', 'Left',   0);
  FormPosTop    := IniFile.ReadInteger('FormCsvText', 'Top',    0);
  FormPosWidth  := IniFile.ReadInteger('FormCsvText', 'Width',  FORM_WIDTH_SET);
  FormPosHeight := IniFile.ReadInteger('FormCsvText', 'Height', FORM_HEIGHT_SET);
  StrFormState  := IniFile.ReadString ('FormCsvText', 'State',  FORM_NORMAL);
  FormPosRight  := FormPosLeft + FormPosWidth;
  FormPosButtom := FormPosTop  + FormPosHeight;
  //j
  IniFile.Free;


  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //Ǎ
  MenuReadAuto.Checked         :=          IniFile.ReadBool   ('Editor', 'ReadAuto',       TRUE);
  MenuReadSJIS.Checked         :=          IniFile.ReadBool   ('Editor', 'ReadSJIS',       FALSE);
  MenuReadJIS.Checked          :=          IniFile.ReadBool   ('Editor', 'ReadJIS',        FALSE);
  MenuReadEUCJP.Checked        :=          IniFile.ReadBool   ('Editor', 'ReadEUCJP',      FALSE);
  MenuReadUTF7.Checked         :=          IniFile.ReadBool   ('Editor', 'ReadUTF7',       FALSE);
  MenuReadUTF8Bom.Checked      :=          IniFile.ReadBool   ('Editor', 'ReadUTF8Bom',    FALSE);
  MenuReadUTF8.Checked         :=          IniFile.ReadBool   ('Editor', 'ReadUTF8',       FALSE);
  MenuReadUTF16BomBe.Checked   :=          IniFile.ReadBool   ('Editor', 'ReadUTF16BomBe', FALSE);
  MenuReadUTF16Be.Checked      :=          IniFile.ReadBool   ('Editor', 'ReadUTF16Be',    FALSE);
  MenuReadUTF16BomLe.Checked   :=          IniFile.ReadBool   ('Editor', 'ReadUTF16BomLe', FALSE);
  MenuReadUTF16Le.Checked      :=          IniFile.ReadBool   ('Editor', 'ReadUTF16Le',    FALSE);
  MenuShowLineNumber.Checked   :=          IniFile.ReadBool   ('Editor', 'ShowLineNumber', FALSE);
  MenuTab2.Checked             :=          IniFile.ReadBool   ('Editor', 'Tab2',  FALSE);
  MenuTab4.Checked             :=          IniFile.ReadBool   ('Editor', 'Tab4',  TRUE);
  MenuTab6.Checked             :=          IniFile.ReadBool   ('Editor', 'Tab6',  FALSE);
  MenuTab8.Checked             :=          IniFile.ReadBool   ('Editor', 'Tab8',  FALSE);
  FontDialog1.Font.Name  :=                IniFile.ReadString ('Editor', 'FontNameReadOnly', 'lr SVbN');
  FontDialog1.Font.Size  := StrToInt      (IniFile.ReadString ('Editor', 'FontSizeReadOnly', '14'));
  FontDialog1.Font.Color := StringToColor (IniFile.ReadString ('Editor', 'FontColorReadOnly', ColorToString(clBlack)));
  ColorDialog1.Color     := StringToColor (IniFile.ReadString ('Editor', 'BackColorReadOnly', ColorToString(clWhite)));
  StrSaveFolderFileCsv         :=          IniFile.ReadString ('Editor', 'FolderFile',   '');
  RadioGroup1.ItemIndex        :=          IniFile.ReadInteger('CsvType', 'Index',     0);
  StrLoadFileName              :=          IniFile.ReadString ('Editor',  'FileName', '');
  IntEditorHwnd                :=          IniFile.ReadInteger('Editor',  'Handle',    0);
  //j
  IniFile.Free;

  //ݒ
  Memo1.Lines.Clear;
  Memo9.Lines.Clear;
  Memo1.Font.Name  := FontDialog1.Font.Name;
  Memo1.Font.Size  := FontDialog1.Font.Size;
  Memo1.Font.Color := FontDialog1.Font.Color;
  Memo1.Color      := ColorDialog1.Color;
  Memo9.Font.Name  := FontDialog1.Font.Name;
  Memo9.Font.Size  := FontDialog1.Font.Size;
  Memo9.Font.Color := FontDialog1.Font.Color;
  Memo9.Color      := ColorDialog1.Color;
  Memo9.Visible    := FALSE;

  //f
  if MenuTab2.Checked = FALSE then
  begin
    TabStops[0] := 4*2;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
  end;
  if MenuTab4.Checked = FALSE then
  begin
    TabStops[0] := 4*4;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
  end;
  if MenuTab6.Checked = FALSE then
  begin
    TabStops[0] := 4*6;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
  end;
  if MenuTab8.Checked = FALSE then
  begin
    TabStops[0] := 4*8;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
  end;

  if MenuShowLineNumber.Checked = TRUE then
  begin
    Memo9.Visible  := TRUE;
    Timer6.Enabled := TRUE;
  end
  else
  begin
    Memo9.Visible  := FALSE;
    Timer6.Enabled := FALSE;
  end;

end;

procedure TFormCsvText.FormDestroy(Sender: TObject);
var
  sStr1: String;
  IniFile: TIniFile;
begin
  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //ۑ
  IniFile.WriteBool   ('Editor',  'ShowLineNumber',    MenuShowLineNumber.Checked);
  IniFile.WriteBool   ('Editor',  'ReadAuto',          MenuReadAuto.Checked);
  IniFile.WriteBool   ('Editor',  'ReadSJIS',          MenuReadSJIS.Checked);
  IniFile.WriteBool   ('Editor',  'ReadJIS',           MenuReadJIS.Checked);
  IniFile.WriteBool   ('Editor',  'ReadEUCJP',         MenuReadEUCJP.Checked);
  IniFile.WriteBool   ('Editor',  'ReadUTF7',          MenuReadUTF7.Checked);
  IniFile.WriteBool   ('Editor',  'ReadUTF8Bom',       MenuReadUTF8Bom.Checked);
  IniFile.WriteBool   ('Editor',  'ReadUTF8',          MenuReadUTF8.Checked);
  IniFile.WriteBool   ('Editor',  'ReadUTF16BomBe',    MenuReadUTF16BomBe.Checked);
  IniFile.WriteBool   ('Editor',  'ReadUTF16Be',       MenuReadUTF16Be.Checked);
  IniFile.WriteBool   ('Editor',  'ReadUTF16BomLe',    MenuReadUTF16BomLe.Checked);
  IniFile.WriteBool   ('Editor',  'ReadUTF16Le',       MenuReadUTF16Le.Checked);
  IniFile.WriteString ('Editor',  'FolderFile',        StrSaveFolderFileCsv);
  IniFile.WriteString ('Editor',  'FontNameReadOnly',  FontDialog1.Font.Name);
  IniFile.WriteString ('Editor',  'FontSizeReadOnly',  IntToStr(FontDialog1.Font.Size));
  IniFile.WriteString ('Editor',  'FontColorReadOnly', ColorToString(FontDialog1.Font.Color));
  IniFile.WriteString ('Editor',  'BackColorReadOnly', ColorToString(ColorDialog1.Color));
  IniFile.WriteString ('Editor',  'FolderFile',        StrSaveFolderFileCsv);
  IniFile.WriteInteger('CsvType', 'Index',             RadioGroup1.ItemIndex);
  //j
  IniFile.Free;

  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //ʏ
  if WindowState = wsMaximized then
  begin
    IniFile.WriteString('FormCsvText', 'State', FORM_MAX);
  end
  else if WindowState = wsMinimized then
  begin
    IniFile.WriteString('FormCsvText', 'State', FORM_MIN);
  end
  else
  begin
    IniFile.WriteString ('FormCsvText', 'State',  FORM_NORMAL);
    IniFile.WriteInteger('FormCsvText', 'Left',   FormCsvText.Left);
    IniFile.WriteInteger('FormCsvText', 'Top',    FormCsvText.Top);
    IniFile.WriteInteger('FormCsvText', 'Width',  FormCsvText.Width);
    IniFile.WriteInteger('FormCsvText', 'Height', FormCsvText.Height);
  end;
  //j
  IniFile.Free;

  //j
  StrListLoadCsvFile.Free;
  StrListLoadFileTmp.Free;
  StrListTextEx.Free;

end;

procedure TFormCsvText.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  //ESC
  if (Shift = []) and (key = VK_ESCAPE) then
  begin
    //I
    Close;
  end
  //
  else
  begin
    ;
  end;
end;

procedure TFormCsvText.FormResize(Sender: TObject);
var
  iWidth1: Integer;
begin
  //f
  iWidth1 := FormCsvText.Width - 640;
  StatusBar1.Panels.Items[0].Width := iWidth1;
  StatusBar1.Panels.Items[1].Width := 100;
  StatusBar1.Panels.Items[2].Width :=  70;
  StatusBar1.Panels.Items[3].Width := 120;
  StatusBar1.Panels.Items[4].Width := 210;
  StatusBar1.Panels.Items[5].Width := 120;
  StatusBar1.Panels.Items[6].Width :=  20;
end;

procedure TFormCsvText.FormShow(Sender: TObject);
var
  sStr1: String;
  iWidth1: Integer;
  IniFile: TIniFile;
begin
  //FormłȂ̂ŁACreatełȂShowōs
  //FormKeyDowñCxgݒ肷
  FormCsvText.KeyPreview := TRUE;

  //GetSystemMetricst֐gpāCzʑŜ̃TCY擾
  MonitorTop    := GetSystemMetrics(SM_YVIRTUALSCREEN);
  MonitorLeft   := GetSystemMetrics(SM_XVIRTUALSCREEN);
  MonitorWidth  := GetSystemMetrics(SM_CXVIRTUALSCREEN);;
  MonitorHeight := GetSystemMetrics(SM_CYVIRTUALSCREEN);;
  MonitorRight  := MonitorLeft + MonitorWidth;
  MonitorButtom := MonitorTop  + MonitorHeight;

  //\ł ȌԂŕ\
  if ( (MonitorTop  <= FormPosTop  + FORM_TOP_SHIFT ) and (FormPosTop    <= MonitorButtom) ) and
     ( (MonitorTop  <= FormPosButtom                ) and (FormPosButtom <= MonitorButtom) ) and
     ( (MonitorLeft <= FormPosLeft + FORM_LEFT_SHIFT) and (FormPosLeft   <= MonitorRight ) ) and
     ( (MonitorLeft <= FormPosRight                 ) and (FormPosRight  <= MonitorRight ) ) then
  begin
    FormCsvText.SetBounds(FormPosLeft + IntFormShiftX, FormPosTop + IntFormShiftY, FormPosWidth, FormPosHeight);
  end
  //ʓɕ\łȂ ftHgŕ\
  else
  begin
    FormCsvText.SetBounds(0, 0 ,FORM_WIDTH_SET, FORM_HEIGHT_SET);
  end;

  //ő
  if StrFormState = FORM_MAX then
  begin
    FormCsvText.WindowState := wsMaximized;
  end
  //ŏ
  else if StrFormState = FORM_MIN then
  begin
    FormCsvText.WindowState := wsMinimized;
  end
  //W
  else
  begin
    //
  end;

  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //Ǎ
  iWidth1   := StrToInt(IniFile.ReadString('LineWidth', 'Text',  '80'));
  //j
  IniFile.Free;

  //ݒ
  Memo9.Width := iWidth1;

  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //Ǎ
  StrReturnMark      := IniFile.ReadString('Editor', 'LineFeed',   '');
  StrCsvSeparater    := IniFile.ReadString('Editor', 'Separater',  '');
  //j
  IniFile.Free;

  //XV
  StatusBar1.Panels.Items[3].Text := '';
  StatusBar1.Panels.Items[4].Text := StrLoadChar;
  StatusBar1.Panels.Items[5].Text := StrLoadReturn;
  StatusBar1.Panels.Items[6].Text := '';

  //ݒ
  BoolFormShow := TRUE;

  //t@Cǂݍ
  Timer1.Enabled := TRUE;
end;

procedure TFormCsvText.MenuCloseClick(Sender: TObject);
begin
  //I
  ModalResult := mrCancel;
end;

procedure TFormCsvText.MenuShowLineNumberClick(Sender: TObject);
begin
  if MenuShowLineNumber.Checked = TRUE then
  begin
    MenuShowLineNumber.Checked  := FALSE;
  end
  else
  begin
    MenuShowLineNumber.Checked  := TRUE;
  end;

  if MenuShowLineNumber.Checked = TRUE then
  begin
    Memo9.Visible  := TRUE;
    Timer6.Enabled := TRUE;
  end
  else
  begin
    Memo9.Visible  := FALSE;
    Timer6.Enabled := FALSE;
  end;
end;

procedure TFormCsvText.MenuTab2Click(Sender: TObject);
begin
  if MenuTab2.Checked = FALSE then
  begin
    MenuTab2.Checked := TRUE;
    MenuTab4.Checked := FALSE;
    MenuTab6.Checked := FALSE;
    MenuTab8.Checked := FALSE;
    TabStops[0] := 4*2;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
    Memo1.Invalidate;
  end;
end;

procedure TFormCsvText.MenuTab4Click(Sender: TObject);
begin
  if MenuTab4.Checked = FALSE then
  begin
    MenuTab2.Checked := FALSE;
    MenuTab4.Checked := TRUE;
    MenuTab6.Checked := FALSE;
    MenuTab8.Checked := FALSE;
    TabStops[0] := 4*4;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
    Memo1.Invalidate;
  end;
end;

procedure TFormCsvText.MenuTab6Click(Sender: TObject);
begin
  if MenuTab6.Checked = FALSE then
  begin
    MenuTab2.Checked := FALSE;
    MenuTab4.Checked := FALSE;
    MenuTab6.Checked := TRUE;
    MenuTab8.Checked := FALSE;
    TabStops[0] := 4*6;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
    Memo1.Invalidate;
  end;
end;

procedure TFormCsvText.MenuTab8Click(Sender: TObject);
begin
  if MenuTab8.Checked = FALSE then
  begin
    MenuTab2.Checked := FALSE;
    MenuTab4.Checked := FALSE;
    MenuTab6.Checked := FALSE;
    MenuTab8.Checked := TRUE;
    TabStops[0] := 4*8;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
    Memo1.Invalidate;
  end;
end;

procedure TFormCsvText.MenuFontColorReadOnlyClick(Sender: TObject);
begin
  //s
  if FontDialog1.Execute = TRUE then
  begin
    //f
    Memo1.Font.Name  := FontDialog1.Font.Name;
    Memo1.Font.Size  := FontDialog1.Font.Size;
    Memo1.Font.Color := FontDialog1.Font.Color;
    //f
    Memo9.Font.Name  := FontDialog1.Font.Name;
    Memo9.Font.Size  := FontDialog1.Font.Size;
    Memo9.Font.Color := FontDialog1.Font.Color;
  end;
end;

procedure TFormCsvText.MenuBackColorReadOnlyClick(Sender: TObject);
begin
  //s
  if ColorDialog1.Execute = TRUE then
  begin
    //f
    Memo1.Color      := ColorDialog1.Color;
    //f
    Memo9.Color      := ColorDialog1.Color;
  end;
end;

procedure TFormCsvText.MenuLineSetSettingClick(Sender: TObject);
var
  sStr1: String;
  iWidth1: Integer;
  bResult: Boolean;
  IniFile: TIniFile;
begin
  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //
  IniFile.WriteString('FormSetting', 'Parent',     PARENT_LINE_NUMBER);
  //j
  IniFile.Free;

  //
  FormSetting := TFormSetting.Create(Application);
  try
    if FormSetting.ShowModal = mrOK then
    begin
      bResult := TRUE;
    end
    else
    begin
      bResult := FALSE;
    end;
  finally
    //j
    FormSetting.Free;
  end;
  Application.ProcessMessages;
  //ݒǍ
  if bResult = TRUE then
  begin
    //m
    sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
    IniFile := TIniFile.Create (sStr1);
    //Ǎ
    iWidth1         := StrToInt(IniFile.ReadString('LineWidth', 'Text',  '80'));
    //j
    IniFile.Free;

    ///ݒ
    Memo9.Width := iWidth1;

    //ݒ
    SetLineNumber(Sender);
    if MenuShowLineNumber.Checked = TRUE then
    begin
      Memo9.Visible  := TRUE;
      Timer6.Enabled := TRUE;
    end
    else
    begin
      Memo9.Visible  := FALSE;
      Timer6.Enabled := FALSE;
    end;
  end;;
end;

procedure TFormCsvText.MenuLfSettingClick(Sender: TObject);
var
  sStr1: String;
  bResult: Boolean;
  IniFile: TIniFile;
begin
  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //
  IniFile.WriteString('FormSetting', 'Parent',     PARENT_ITEM_LF);
  //j
  IniFile.Free;

  //
  FormSetting := TFormSetting.Create(Application);
  try
    if FormSetting.ShowModal = mrOK then
    begin
      bResult := TRUE;
    end
    else
    begin
      bResult := FALSE;
    end;
  finally
    //j
    FormSetting.Free;
  end;
  Application.ProcessMessages;
  //ݒǍ
  if bResult = TRUE then
  begin
    //m
    sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
    IniFile := TIniFile.Create (sStr1);
    //Ǎ
    StrReturnMark      := IniFile.ReadString('Editor', 'LineFeed',   '');
    StrCsvSeparater    := IniFile.ReadString('Editor', 'Separater',  '');
    //j
    IniFile.Free;

    //Ǎ
    LoadFileAuto(Sender);
  end;;
end;

procedure TFormCsvText.MenuReadSJISClick(Sender: TObject);
begin
  if MenuReadSJIS.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := TRUE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;;
end;

procedure TFormCsvText.MenuReadJISClick(Sender: TObject);
begin
  if MenuReadJIS.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := TRUE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.MenuReadEUCJPClick(Sender: TObject);
begin
  if MenuReadEUCJP.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := TRUE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.MenuReadUTF7Click(Sender: TObject);
begin
  if MenuReadUTF7.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := TRUE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.MenuReadUTF8BomClick(Sender: TObject);
begin
  if MenuReadUTF8Bom.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := TRUE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.MenuReadUTF16BomBeClick(Sender: TObject);
begin
  if MenuReadUTF16BomBe.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := TRUE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.MenuReadUTF16BeClick(Sender: TObject);
begin
  if MenuReadUTF16Be.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := TRUE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.MenuReadUTF16BomLeClick(Sender: TObject);
begin
  if MenuReadUTF16BomLe.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := TRUE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.MenuReadUTF8Click(Sender: TObject);
begin
  if MenuReadUTF8.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := TRUE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.MenuReadUTF16LeClick(Sender: TObject);
begin
  if MenuReadUTF16Le.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := FALSE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := TRUE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.MenuReadAutoClick(Sender: TObject);
begin
  if MenuReadAuto.Checked = FALSE then
  begin
    //ݒ
    MenuReadAuto.Checked       := TRUE;
    MenuReadSJIS.Checked       := FALSE;
    MenuReadJIS.Checked        := FALSE;
    MenuReadEUCJP.Checked      := FALSE;
    MenuReadUTF7.Checked       := FALSE;
    MenuReadUTF8Bom.Checked    := FALSE;
    MenuReadUTF8.Checked       := FALSE;
    MenuReadUTF16BomBe.Checked := FALSE;
    MenuReadUTF16Be.Checked    := FALSE;
    MenuReadUTF16BomLe.Checked := FALSE;
    MenuReadUTF16Le.Checked    := FALSE;
    //Ǎ
    LoadFileAuto(Sender);
  end;
end;

procedure TFormCsvText.Memo1KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  //XV
  MemoStatus(Sender);
end;

procedure TFormCsvText.Memo9Click(Sender: TObject);
begin
  Memo1.SetFocus;
end;

function TFormCsvText.GetcurrentFileName(hwnd: THandle): String;
var
  szName: array [0..1023] of Char;
begin
  //擾
  Editor_Info(hwnd, MI_GET_FILE_NAME, LPARAM(@szName));
  //
  Result := StrPas(szName);

// Editor_Info
//   ̏擾Eݒ肵܂
// p[^
//   hwnd: EBhẼnh
//   nCmd: 擾Eݒ肷̎
// ߂l
//   nCmdɂقȂ܂
// l
//   MI_GET_FILE_NAME:      JĂt@C擾܂

//function Editor_Info(hwnd: THandle; nCmd: WPARAM; lParam: LPARAM): LRESULT;
//begin
//  Result := LRESULT(SendMessage(hwnd, ME_INFO, nCmd, lParam));
//end;
end;

procedure TFormCsvText.ButtonShowMeryClick(Sender: TObject);
var
  sStr1: String;
  sStr2: String;
  iIndex1 : Integer;
  bTab: Boolean;
  IniFile: TIniFile;
begin
  //tH_
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + CONVERT_FOLDER_NAME;
  //t@C
  sStr2 := sStr1 + '\' + CONVERT_FILE_NAME;
  //t@Cۑ
  Memo1.Lines.SaveToFile(sStr2, Tencoding.Unicode);


  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //Ǎ
  iIndex1 := IniFile.ReadInteger('TabEnabled',  'Index',         0);
  //j
  IniFile.Free;

  // -----------------------------------------------------------------------------
// Editor_Info
//   ̏擾Eݒ肵܂
// p[^
//   hwnd: EBhẼnh
//   nCmd: 擾Eݒ肷̎
// ߂l
//   nCmdɂقȂ܂
// l
//   MI_IS_WINDOW_COMBINED: ^u̗LE擾܂

  bTab := Boolean(Editor_Info(IntEditorHwnd, MI_IS_WINDOW_COMBINED, 0));
  //^uL
  if bTab = TRUE then
  begin
    //ʃEBhE
    if iIndex1 = 0 then
    begin

// -----------------------------------------------------------------------------
// Editor_LoadFile
//   t@CJ܂
// p[^
//   hwnd:            EBhẼnh
//   bAllowNewWindow: VEBhEŊJ܂
//   szFileName:      ǂݍރt@C
// ߂l
//   TrueԂ܂
// -----------------------------------------------------------------------------
      //t@CJ
      if Editor_LoadFile(IntEditorHwnd, TRUE, PWideChar(sStr2)) = FALSE then
      begin
        //m
        sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
        IniFile := TIniFile.Create (sStr1);
        //
        IniFile.WriteBool('Editor', 'LoadFile', FALSE);
        //j
        IniFile.Free;
      end;
    end
    //^u
    else
    begin
      //
    end;
  end;

  //I
  ModalResult := mrOK;
end;

procedure TFormCsvText.ButtonSaveClick(Sender: TObject);
var
  sStr1: String;
  sStr2: String;
  sStr3: String;
  sStr4: String;
  iRow1: Integer;
  iPos1: Integer;
  StrList1: TStringList;
begin
  //mF
  if Memo1.Lines.Count = 0 then
  begin
    showmessage('eLXgǂݍ܂Ă܂');
    Exit;
  end;

  //ݒ
  SaveDialog1.Filter       := 'Text(*.txt;*.csv)|*.txt;*.csv|All(*.*)|*.*';
  SaveDialog1.FileName     := ExtractFileName(StrSaveFolderFileCsv);
  SaveDialog1.InitialDir   := ExtractFilePath(StrSaveFolderFileCsv);
  if SaveDialog1.Execute = TRUE then
  begin
    //
    if SaveDialog1.FileName = '' then
    begin
      showmessage('t@Cݒ肳Ă܂');
      Exit;
    end
    else
    begin
      //ݒ
      StrSaveFolderFileCsv := SaveDialog1.FileName;

      //m
      StrList1 := TStringList.Create;

      //s
      for iRow1 := 0 to Memo1.Lines.Count-1 do
      begin
        //ݒ
        sStr1 := '';
        sStr2 := Memo1.Lines.Strings[iRow1];
        //
        while TRUE do
        begin
          //
          iPos1 := Pos(StrCsvSeparater, sStr2);
          if iPos1 > 0 then
          begin
            sStr3 := Copy(sStr2, 1, iPos1-1);
            sStr4 := Copy(sStr2, iPos1+1, Length(sStr2));
            sStr2 := sStr4;
            //ǉ
            sStr1 := sStr1 + sStr3 + #09;
          end
          else
          begin
            //ǉ
            sStr1 := sStr1 + sStr2;
            //f
            Break;
          end;
        end;
        //ǉ
        Strlist1.Add(sStr1);
      end;
      //ۑ
      StrList1.SaveToFile(StrSaveFolderFileCsv, TEncoding.Unicode);

      //j
      StrList1.Free;

      //\
      showmessage('t@Cۑ܂');
    end;
  end;
end;

procedure TFormCsvText.ButtonCancelClick(Sender: TObject);
begin
  //I
  ModalResult := mrCancel;
end;

procedure TFormCsvText.RadioGroup1Click(Sender: TObject);
begin
  LoadFileAuto(Sender);
end;

procedure TFormCsvText.Timer1Timer(Sender: TObject);
begin
  //Zbg
  Timer1.Enabled := FALSE;

  //\
  LoadFileAuto(Sender);
end;

procedure TFormCsvText.Timer6Timer(Sender: TObject);
var
  iLine5: Integer;
  iLine6: Integer;
begin

  if Memo1.Lines.Count > 0 then
  begin
    //ʂ̍ŏs
    iLine5 := Memo1.Perform(EM_GETFIRSTVISIBLELINE, 0, 0);
    //OƈႤ
    if iLine5 <> IntTopLineMemo then
    begin
      //ʂ̍ŏs
      iLine6 := Memo9.Perform(EM_GETFIRSTVISIBLELINE, 0, 0);
      //ړ
      Memo9.Perform(EM_LINESCROLL, 0, iLine5 - iLine6);
    end;
    IntTopLineMemo := iLine5;
  end;
end;

procedure TFormCsvText.SetLineNumber(Sender: TObject);
var
  sStr1:String;
  iRow1: Integer;
  iEnd1: Integer;
  iIndex1: Integer;
  IniFile: TIniFile;
  StrList1: TStringList;
  StrStream1: TStringStream;
begin
  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create(sStr1);
  //Ǎ
  iIndex1  := IniFile.ReadInteger('LineNumber', 'LineZero',    1);
  //j
  IniFile.Free;

  // EOF ̕mۂ
  iEnd1 := Memo1.Lines.Count;

  //ݒ
  StrList1   := TStringList.Create;
  if iindex1 = 0 then
  begin
    for iRow1 := 0 to iEnd1 do
    begin
      StrList1.Add(Format('%6d', [iRow1+1]))
    end;
    //t@C܂̓Xg[͍ɕ\\
    StrStream1 := TStringStream.Create(StrList1.Text);
    Memo9.Lines.BeginUpdate;
    Memo9.Lines.LoadFromStream(StrStream1);
    Memo9.Lines.EndUpdate;
    StrStream1.Free;
  end
  else
  begin
    for iRow1 := 0 to iEnd1 do
    begin
      StrList1.Add(Format('%.6d', [iRow1+1]))
    end;
    //t@C܂̓Xg[͍ɕ\\
    StrStream1 := TStringStream.Create(StrList1.Text);
    Memo9.Lines.BeginUpdate;
    Memo9.Lines.LoadFromStream(StrStream1);
    Memo9.Lines.EndUpdate;
    StrStream1.Free;
  end;
  StrList1.Free;
end;

procedure TFormCsvText.MenuShowMerySettingClick(Sender: TObject);
var
  sStr1: String;
  bResult: Boolean;
  IniFile: TIniFile;
begin
  //m
  sStr1 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + INIFILE_NAME;
  IniFile := TIniFile.Create (sStr1);
  //
  IniFile.WriteString('FormSetting', 'Parent',     PARENT_SHOW_MERY);
  //j
  IniFile.Free;

  //
  FormSetting := TFormSetting.Create(Application);
  try
    if FormSetting.ShowModal = mrOK then
    begin
      bResult := TRUE;
    end
    else
    begin
      bResult := FALSE;
    end;
  finally
    //j
    FormSetting.Free;
  end;
  Application.ProcessMessages;
  //ݒǍ
  if bResult = TRUE then
  begin
    //
  end;;
end;

procedure TFormCsvText.LoadCsvFile(Sender: TObject);
var
  pBuff1: PAnsiChar;
  pAnsiChar1: PAnsiChar;
  sjis: SJISString;
  jis: JISString;
  utf7: UTF7String;
  utf8: UTF8String;
  eucjp: EUCString;
  iRow1: Integer;
  iRow2: Integer;
  iStart1: Integer;
  iEnd1: Integer;
  iCopyCnt: Integer;
  hFindData1: THandle;
  File1: TSearchRec;
  FileStream1 :TFileStream;
begin
  //mF
  if StrLoadFileName = '' then
  begin
    showmessage('t@Cǂݍ܂Ă܂');
    Exit;
  end;
  //t@C擾
  hFindData1 := FindFirst(StrLoadFileName, faAnyFile, File1);
  try
    ;
  finally
    FindClose(File1);
  end;
  //s
  if hFindData1 <> 0 then
  begin
    ShowMessage('t@C܂P');
    Exit;
  end;
  //t@CTCY
  IntLoadFileSize := File1.Size;

  //\
  StatusBar1.Panels.Items[3].Text := '';
  StatusBar1.Panels.Items[4].Text := '';
  StatusBar1.Panels.Items[5].Text := '';
  StatusBar1.Panels.Items[6].Text := '';

  //
  IntReadEncode1 := ENCODE_UNKNOWN;
  IntReadReturn1 := RETURN_NONE;

  //͎o
  if MenuReadAuto.Checked = TRUE then
  begin
    //擾
    GetCharCodeRetCode(StrLoadFileName, IntReadEncode1, IntReadReturn1, Sender);
  end
  //͐ݒɏ]
  else
  begin
    //擾 spɎs
    GetCharCodeRetCode(StrLoadFileName, IntReadEncode1, IntReadReturn1, Sender);
    //ݒ
    if      MenuReadSJIS.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_SJIS;
    end
    else if MenuReadJIS.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_JIS;
    end
    else if MenuReadEUCJP.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_EUCJP;
    end
    else if MenuReadUTF7.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF7;
    end
    else if MenuReadUTF8Bom.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF8_BOM_ON;
    end
    else if MenuReadUTF8.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF8_BOM_OFF;
    end
    else if MenuReadUTF16BomBe.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF16_BOM_ON_BE;
    end
    else if MenuReadUTF16Be.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF16_BOM_OFF_BE;
    end
    else if MenuReadUTF16BomLe.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF16_BOM_ON_LE;
    end
    else if MenuReadUTF16Le.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF16_BOM_OFF_LE;
    end
    else
    begin
      IntReadEncode1 := ENCODE_UNKNOWN;
    end;
  end;

  //sɉs
  if BoolLfInLine = TRUE then
  begin
    //\
    showmessage('s(CrLf)܂łɁAɉs(Lf)܂');
  end;

  //NA
  StrListLoadCsvFile.Clear;

  //UnKnown
  if BoolUnKnownCharCode = TRUE then
  begin
    showmessage('sȕ܂ł܂̂ŕ\ł܂');
    StrLoadChar := CHAR_BINARY;
  end
  //SJIS
  else if IntReadEncode1 = ENCODE_SJIS then
  begin
    StrLoadChar := CHAR_SJIS;
  end
  //JIS
  else if IntReadEncode1 = ENCODE_JIS then
  begin
    StrLoadChar := CHAR_JIS;
  end
  //EUCJP
  else if IntReadEncode1 = ENCODE_EUCJP then
  begin
    StrLoadChar := CHAR_EUCJP;
  end
  //UTF7
  else if IntReadEncode1 = ENCODE_UTF7 then
  begin
    StrLoadChar := CHAR_UTF7;;
  end
  //UTF8 BOM
  else if IntReadEncode1 = ENCODE_UTF8_BOM_ON then
  begin
    StrLoadChar := CHAR_UTF8_BOM_ON;
  end
  //UTF8
  else if IntReadEncode1 = ENCODE_UTF8_BOM_OFF then
  begin
    StrLoadChar := CHAR_UTF8_BOM_OFF;
  end
  //UTF16BOM BE
  else if IntReadEncode1 = ENCODE_UTF16_BOM_ON_BE then
  begin
    StrLoadChar := CHAR_UTF16_BOM_ON_BE;
  end
  //UTF16 BE
  else if IntReadEncode1 = ENCODE_UTF16_BOM_OFF_BE then
  begin
    StrLoadChar := CHAR_UTF16_BOM_OFF_BE;
  end
  //UTF16BOM LE
  else if IntReadEncode1 = ENCODE_UTF16_BOM_ON_LE then
  begin
    StrLoadChar := CHAR_UTF16_BOM_ON_LE;
  end
  //UTF16 LE
  else if IntReadEncode1 = ENCODE_UTF16_BOM_OFF_LE then
  begin
    StrLoadChar := CHAR_UTF16_BOM_OFF_LE;
  end
  //͂Ȃ
  else
  begin
    StrLoadChar := CHAR_UNKNOWN;
  end;

  //CR+LF
  if IntReadReturn1 = RETURN_CRLF then
  begin
    StrLoadReturn := RET_CRLF;
  end
  //CR
  else if IntReadReturn1 = RETURN_CR then
  begin
    StrLoadReturn := RET_CR;
  end
  //LF
  else if IntReadReturn1 = RETURN_LF then
  begin
    StrLoadReturn := RET_LF;
  end
  //NONE
  else if IntReadReturn1 = RETURN_NONE then
  begin
    StrLoadReturn := RET_NONE;
  end
  //NONE
  else
  begin
    StrLoadReturn := RET_UNKNOWN;
  end;

  //֎~ ܂ s
  if (Bool(File1.Attr and FILE_ATTRIBUTE_READONLY) = TRUE) or (BoolUnKnownCharCode = TRUE) then
  begin
    //ݒ
    BoolFileReadOnly := TRUE;
    //\
    StatusBar1.Panels.Items[3].Text := FILE_READ_ONLY;
    StatusBar1.Panels.Items[4].Text := StrLoadChar;
    StatusBar1.Panels.Items[5].Text := StrLoadReturn;
    StatusBar1.Panels.Items[6].Text := '';
  end
  else
  begin
    //ݒ
    BoolFileReadOnly := FALSE;
    //\
    StatusBar1.Panels.Items[3].Text := '';
    StatusBar1.Panels.Items[4].Text := StrLoadChar;
    StatusBar1.Panels.Items[5].Text := StrLoadReturn;
    StatusBar1.Panels.Items[6].Text := '';
  end;


  //TCYw肵ăm
  pBuff1 := AllocMem(File1.Size);
  try
    //ǂݍ
    try
      //̓t@Cݒ肵ēǂݍ
      FileStream1 := TFileStream.Create(StrLoadFileName, fmOpenRead);
      try
        FileStream1.Read(pBuff1^, File1.Size);
      finally
        FileStream1.Free;
      end;

      //͈͂w
      //UTF8 BOM
      if IntReadEncode1 = ENCODE_UTF8_BOM_ON then
      begin
        iStart1 := 3;
        iEnd1   := File1.Size;
      end
      //UTF16BOM BE
      else if IntReadEncode1 = ENCODE_UTF16_BOM_ON_BE then
      begin
        iStart1 := 2;
        iEnd1   := File1.Size;
      end
      //UTF16BOM LE
      else if IntReadEncode1 = ENCODE_UTF16_BOM_ON_LE then
      begin
        iStart1 := 2;
        iEnd1   := File1.Size;
      end
      //ȊO
      else
      begin
        iStart1 := 0;
        iEnd1   := File1.Size;
      end;

      //UTF16 LE
      if      (IntReadEncode1 = ENCODE_UTF16_BOM_ON_LE) or (IntReadEncode1 = ENCODE_UTF16_BOM_OFF_LE) then
      begin
        //2ɏ̂ for ͎gȂ
        iRow1 := iStart1;
        while iRow1 < iEnd1 do
        begin
          //CR+Lf
          if IntReadReturn1 = RETURN_CRLF then
          begin
            if      ( (pBuff1[iRow1+0] = #13) and (pBuff1[iRow1+1] = #00) and (pBuff1[iRow1+2] = #10) and (pBuff1[iRow1+3] = #00) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //4̏
              iRow1 := iRow1 + 4;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 2;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //4̏
              iRow1 := iRow1 + 4;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //CR
          else if IntReadReturn1 = RETURN_CR then
          begin
            if      ( (pBuff1[iRow1+0] = #13) and (pBuff1[iRow1+1] = #00) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 2;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //LF
          else if IntReadReturn1 = RETURN_LF then
          begin
            if      ( (pBuff1[iRow1+0] = #10) and (pBuff1[iRow1+1] = #00) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 2;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //NONE
          else if IntReadReturn1 = RETURN_NONE then
          begin
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end;
          end;
        end;
      end
      //UTF16 BE
      else if (IntReadEncode1 = ENCODE_UTF16_BOM_ON_BE) or (IntReadEncode1 = ENCODE_UTF16_BOM_OFF_BE) then
      begin
        //2ɏ̂ for ͎gȂ
        iRow1 := iStart1;
        while iRow1 < iEnd1 do
        begin
          //CR+Lf
          if IntReadReturn1 = RETURN_CRLF then
          begin
            if      ( (pBuff1[iRow1+0] = #00) and (pBuff1[iRow1+1] = #13) and (pBuff1[iRow1+2] = #00) and (pBuff1[iRow1+3] = #10) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                if (iRow2 mod 2) = 0 then
                begin
                  pAnsiChar1[iRow2+1] := pBuff1[iStart1+iRow2];
                end
                else
                begin
                  pAnsiChar1[iRow2-1] := pBuff1[iStart1+iRow2];
                end;
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //4̏
              iRow1 := iRow1 + 4;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 2;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                if (iRow2 mod 2) = 0 then
                begin
                  pAnsiChar1[iRow2+1] := pBuff1[iStart1+iRow2];
                end
                else
                begin
                  pAnsiChar1[iRow2-1] := pBuff1[iStart1+iRow2];
                end;
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //4̏
              iRow1 := iRow1 + 4;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //CR
          else if IntReadReturn1 = RETURN_CR then
          begin
            if      ( (pBuff1[iRow1+0] = #00) and (pBuff1[iRow1+1] = #13) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                if (iRow2 mod 2) = 0 then
                begin
                  pAnsiChar1[iRow2+1] := pBuff1[iStart1+iRow2];
                end
                else
                begin
                  pAnsiChar1[iRow2-1] := pBuff1[iStart1+iRow2];
                end;
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 2;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                if (iRow2 mod 2) = 0 then
                begin
                  pAnsiChar1[iRow2+1] := pBuff1[iStart1+iRow2];
                end
                else
                begin
                  pAnsiChar1[iRow2-1] := pBuff1[iStart1+iRow2];
                end;
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //LF
          else if IntReadReturn1 = RETURN_LF then
          begin
            if      ( (pBuff1[iRow1+0] = #00) and (pBuff1[iRow1+1] = #10) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                if (iRow2 mod 2) = 0 then
                begin
                  pAnsiChar1[iRow2+1] := pBuff1[iStart1+iRow2];
                end
                else
                begin
                  pAnsiChar1[iRow2-1] := pBuff1[iStart1+iRow2];
                end;
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 2;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                if (iRow2 mod 2) = 0 then
                begin
                  pAnsiChar1[iRow2+1] := pBuff1[iStart1+iRow2];
                end
                else
                begin
                  pAnsiChar1[iRow2-1] := pBuff1[iStart1+iRow2];
                end;
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //NONE
          else if IntReadReturn1 = RETURN_NONE then
          begin
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+2);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                if (iRow2 mod 2) = 0 then
                begin
                  pAnsiChar1[iRow2+1] := pBuff1[iStart1+iRow2];
                end
                else
                begin
                  pAnsiChar1[iRow2-1] := pBuff1[iStart1+iRow2];
                end;
              end;
              pAnsiChar1[iCopyCnt+0] := #00;
              pAnsiChar1[iCopyCnt+1] := #00;
              //ǉ
              StrListLoadCsvFile.Add(WideString(PWideChar(@pAnsiChar1[0])));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+2);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end;
          end;
        end;
      end
      //UTF8
      else if (IntReadEncode1 = ENCODE_UTF8_BOM_ON) or (IntReadEncode1 = ENCODE_UTF8_BOM_OFF) then
      begin
        //2ɏ̂ for ͎gȂ
        iRow1 := iStart1;
        while iRow1 < iEnd1 do
        begin
          //CR+Lf
          if IntReadReturn1 = RETURN_CRLF then
          begin
            if      ( (pBuff1[iRow1+0] = #13) and (pBuff1[iRow1+1] = #10) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              utf8 := pAnsiChar1;
              StrListLoadCsvFile.Add(WideString(utf8));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 1;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              utf8 := pAnsiChar1;
              StrListLoadCsvFile.Add(WideString(utf8));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //CR
          else if IntReadReturn1 = RETURN_CR then
          begin
            if      ( (pBuff1[iRow1+0] = #13) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              utf8 := pAnsiChar1;
              StrListLoadCsvFile.Add(WideString(utf8));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 1;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              utf8 := pAnsiChar1;
              StrListLoadCsvFile.Add(WideString(utf8));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //LF
          else if IntReadReturn1 = RETURN_LF then
          begin
            if      ( (pBuff1[iRow1+0] = #10)) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              utf8 := pAnsiChar1;
              StrListLoadCsvFile.Add(WideString(utf8));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 1;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              utf8 := pAnsiChar1;
              StrListLoadCsvFile.Add(WideString(utf8));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //NONE
          else if IntReadReturn1 = RETURN_NONE then
          begin
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              utf8 := pAnsiChar1;
              StrListLoadCsvFile.Add(WideString(utf8));
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end;
          end;
        end;
      end
      //ȊO
      else
      begin
        //2ɏ̂ for ͎gȂ
        iRow1 := iStart1;
        while iRow1 < iEnd1 do
        begin
          //CR+Lf
          if IntReadReturn1 = RETURN_CRLF then
          begin
            if      ( (pBuff1[iRow1+0] = #13) and (pBuff1[iRow1+1] = #10) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              //SJIS
              if IntReadEncode1 = ENCODE_SJIS then
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_JIS then
              begin
                jis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(jis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_EUCJP then
              begin
                eucjp := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(eucjp));
              end
              //UTF7
              else if IntReadEncode1 = ENCODE_UTF7 then
              begin
                utf7 := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(utf7));
              end
              //
              else
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end;
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 1;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              //SJIS
              if IntReadEncode1 = ENCODE_SJIS then
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_JIS then
              begin
                jis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(jis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_EUCJP then
              begin
                eucjp := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(eucjp));
              end
              //UTF7
              else if IntReadEncode1 = ENCODE_UTF7 then
              begin
                utf7 := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(utf7));
              end
              //
              else
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end;
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //2̏
              iRow1 := iRow1 + 2;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //CR
          else if IntReadReturn1 = RETURN_CR then
          begin
            if      ( (pBuff1[iRow1+0] = #13) ) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              //SJIS
              if IntReadEncode1 = ENCODE_SJIS then
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_JIS then
              begin
                jis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(jis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_EUCJP then
              begin
                eucjp := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(eucjp));
              end
              //UTF7
              else if IntReadEncode1 = ENCODE_UTF7 then
              begin
                utf7 := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(utf7));
              end
              //
              else
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end;
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 1;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              //SJIS
              if IntReadEncode1 = ENCODE_SJIS then
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_JIS then
              begin
                jis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(jis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_EUCJP then
              begin
                eucjp := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(eucjp));
              end
              //UTF7
              else if IntReadEncode1 = ENCODE_UTF7 then
              begin
                utf7 := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(utf7));
              end
              //
              else
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end;
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //LF
          else if IntReadReturn1 = RETURN_LF then
          begin
            if      ( (pBuff1[iRow1+0] = #10)) then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              //SJIS
              if IntReadEncode1 = ENCODE_SJIS then
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_JIS then
              begin
                jis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(jis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_EUCJP then
              begin
                eucjp := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(eucjp));
              end
              //UTF7
              else if IntReadEncode1 = ENCODE_UTF7 then
              begin
                utf7 := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(utf7));
              end
              //
              else
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end;
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end
            else
            begin
              //2̏
              iRow1 := iRow1 + 1;
            end;
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              //SJIS
              if IntReadEncode1 = ENCODE_SJIS then
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_JIS then
              begin
                jis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(jis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_EUCJP then
              begin
                eucjp := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(eucjp));
              end
              //UTF7
              else if IntReadEncode1 = ENCODE_UTF7 then
              begin
                utf7 := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(utf7));
              end
              //
              else
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end;
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end;
          end
          //NONE
          else if IntReadReturn1 = RETURN_NONE then
          begin
            //Ō
            if iRow1 = iEnd1-1 then
            begin
              //oCg
              iCopyCnt := iRow1 - iStart1;
              //m
              pAnsiChar1 := AllocMem(iCopyCnt+1);
              //]
              for iRow2 := 0 to iCopyCnt-1 do
              begin
                pAnsiChar1[iRow2] := pBuff1[iStart1+iRow2];
              end;
              pAnsiChar1[iCopyCnt] := #00;
              //ǉ
              //SJIS
              if IntReadEncode1 = ENCODE_SJIS then
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_JIS then
              begin
                jis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(jis));
              end
              //EUCJP
              else if IntReadEncode1 = ENCODE_EUCJP then
              begin
                eucjp := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(eucjp));
              end
              //UTF7
              else if IntReadEncode1 = ENCODE_UTF7 then
              begin
                utf7 := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(utf7));
              end
              //
              else
              begin
                sjis := pAnsiChar1;
                StrListLoadCsvFile.Add(WideString(sjis));
              end;
              //j
              FreeMem(pAnsiChar1, iCopyCnt+1);
              //1̏
              iRow1 := iRow1 + 1;
              //Jnʒu
              istart1 := iRow1;
            end;
          end;
        end;
      end;
    except
      //ReadOnly J 103 ̃G[ɂȂ΍
    end;
  finally
    FreeMem(pBuff1);
  end;

end;




procedure TFormCsvText.StringToCsvList(SrcString: String; CsvList: TstringList);
var
  sStr1: String;
  sSearch: String;
  sAfter: String;
  iPos1: Integer;
  iPos2: Integer;
  iPos3: Integer;
begin
  Application.ProcessMessages;
  //f[^̐ݒ
  sStr1 := SrcString;
  //^ũf[^
  iPos1 := Pos(string(char($09)), sStr1);
  if iPos1 > 0 then
  begin
    while iPos1 > 0 do
    begin
      //TAB ȑO
      sSearch := Copy(sStr1, 1, iPos1-1);
      //TAB ȍ~
      sAfter := Copy(sStr1, iPos1+1, Length(sStr1));
      //Xgɒǉ
      CsvList.Add(sSearch);
      //Đݒ肵čČ
      sStr1 := sAfter;
      iPos1 := Pos(string(char($09)), sStr1);
    end;
    //Ō̕Xgɒǉ
    CsvList.Add(sStr1);
  end

  //J}dp̃f[^
  else
  begin
    //Đ擪ɂ dp̃f[^
    iPos1 := Pos('"', sStr1);
    if iPos1 = 1 then
    begin
      //" ؂
      //" ȍ~
      sAfter := Copy(sStr1, 2, Length(sStr1));
      //ēxčŌɂ
      iPos2 := Pos('"', sAfter);
      if iPos2 = Length(sAfter) then
      begin
        //dpP̃f[^
        //" ȍ~ " ȑO
        sSearch := Copy(sStr1, 2, Length(sStr1)-2);
        //Xgɒǉ
        CsvList.Add(sSearch);
      end

      //ēxčŌɂȂ
      else
      begin
        //dp̃f[^
        //" ȍ~
        sAfter := Copy(sStr1, 2, Length(sStr1));
        //Đݒ肵čČ
        sStr1 := sAfter;
        iPos2 := Pos('","', sStr1);
        while (iPos2 > 0) do
        begin
          //" ȍ~ " ȑO
          sSearch := Copy(sStr1, 1, iPos2-1);
          //", ȍ~
          sAfter := Copy(sStr1, iPos2+3, Length(sStr1));
          //Xgɒǉ
          CsvList.Add(sSearch);
          //Đݒ肵čČ
          sStr1 := sAfter;
          iPos2 := Pos('","', sStr1);
        end;
        //Ō̕Xgɒǉ
        //" ȑO
        sSearch := Copy(sStr1, 1, Length(sStr1)-1);
        CsvList.Add(sSearch);
      end;
    end

    //ĂȂ J}؂̃f[^
    else
    begin
      //" ؂
      //ĂȂ
      iPos3 := Pos(',', sStr1);
      if iPos3 = 0 then
      begin
        //P̃f[^
        //̂܂
        sSearch := Copy(sStr1, 1, Length(sStr1));
        //Xgɒǉ
        CsvList.Add(sSearch);
      end
      //Ă
      else
      begin
        //Ō܂Ō
        while (iPos3 > 0) do
        begin
          //, ȑO
          sSearch := Copy(sStr1, 1, iPos3-1);
          //, ȍ~
          sAfter := Copy(sStr1, iPos3+1, Length(sStr1));
          //Xgɒǉ
          CsvList.Add(sSearch);
          //Đݒ肵čČ
          sStr1 := sAfter;
          iPos3 := Pos(',', sStr1);
        end;
        //Ō̕Xgɒǉ
        CsvList.Add(sStr1);
      end;
    end;
  end;
end;

procedure TFormCsvText.StringToCsvListEx1(SrcString: String; CsvList: TstringList);
var
  sStr1: String;
  sStr2: String;
  sStr3: String;
  sStr4: String;
  sStr5: String;
  iPos1: Integer;
  iPos2: Integer;
  iRow1: Integer;
  iRow2: Integer;
  iVal1: Integer;
  iVal2: Integer;
  iVal3: Integer;
  bFind1: Boolean;
  StrList1: TStringList;
  StrList2: TStringList;
  StrList3: TStringList;
  StrList4: TStringList;
begin
  //m
  StrList1 := TStringList.Create;
  StrList2 := TStringList.Create;
  StrList3 := TStringList.Create;
  StrList4 := TStringList.Create;

// 11,"22,22",33
// 1,2,3
// "11,22"
// 11,"22,""22",33

  //f[^̐ݒ
  //擪ɒǉ
  sStr1 := ',' + SrcString;

  //ŏɒu
  sStr2 := StringReplace(sStr1, '""', #$0C, [rfReplaceAll]);

  //
  bFind1 := FALSE;
  for iRow1 := 0 to Length(sStr2)-1 do
  begin
    if Copy(sStr2, iRow1+1, 1) = '"' then
    begin
      if bFind1 = FALSE then
      begin
        StrList1.Add(IntToStr(iRow1));
        bFind1 := TRUE;
      end
      else
      begin
        StrList2.Add(IntToStr(iRow1));
        bFind1 := FALSE;
      end;
    end
    else if Copy(sStr2, iRow1+1, 1) = ',' then
    begin
        StrList3.Add(IntToStr(iRow1));
    end
    else
    begin
      //
    end;
  end;

  //2dp̊ԂɂJ}폜
  for iRow1 := 0 to StrList3.Count-1 do
  begin
    //ڐ
    if StrList1.Count = StrList2.Count then
    begin
      if StrList1.Count = 0 then
      begin
            StrList4.Add(StrList3.Strings[iRow1]);
      end
      else
      begin
        //ԍ
        iVal1 := StrToInt(StrList3.Strings[iRow1]);
        for iRow2 := 0 to StrList1.Count-1 do
        begin
          //ԍ
          iVal2 := StrToInt(StrList1.Strings[iRow2]);
          //ԍ
          iVal3 := StrToInt(StrList2.Strings[iRow2]);
          //͈͓
          if (iVal1 > iVal2) and (iVal1 < iVal3)  then
          begin
            //
          end
          else
          begin
            StrList4.Add(StrList3.Strings[iRow1]);
          end;
        end;
      end;
    end;
  end;
          StrList4.Add(IntToStr(Length(sStr2)));

  //ɖ߂
  sStr3 := StringReplace(sStr2, #$0C, '"', [rfReplaceAll]);

  if StrList4.Count = 1 then
  begin
      //擾
      iPos1 := 0;
      iPos2 := StrToInt(StrList4.Strings[0]);
      sStr4 := Copy(sStr3, iPos1+2, iPos2 - iPos1 - 1);
      if Length(sStr4) > 0 then
      begin
        //ǉ
        CsvList.Add(sStr4);
      end;
  end
  else
  begin
    //ݒ
    for iRow1 := 0 to StrList4.Count-2 do
    begin
      //擾
      iPos1 := StrToInt(StrList4.Strings[iRow1+0]);
      iPos2 := StrToInt(StrList4.Strings[iRow1+1]);
      sStr4 := Copy(sStr3, iPos1+2, iPos2 - iPos1 - 1);
      if Length(sStr4) > 2 then
      begin
        if (Copy(sStr4, 1, 1) = '"') and (Copy(sStr4, Length(sStr4), 1) = '"') then
        begin
          sStr5 := Copy(sStr4, 2, Length(sStr4)-2);
        end
        else
        begin
          sStr5 := sStr4;
        end;
      end
      else
      begin
          sStr5 := sStr4;
      end;
      //ǉ
      CsvList.Add(sStr5);
    end;
  end;

  //j
  StrList1.Free;
  StrList2.Free;
  StrList3.Free;
  StrList4.Free;

end;

procedure TFormCsvText.GetCharCodeRetCode(FileName:String; var CharCode: Integer; var RetCode: Integer; Sender: TObject);
var
  sStr2: String;
  sStr3: String;
  i: Integer;
  iCr1: Integer;
  iLf1: Integer;
  iMax1: Integer;
  bSJIS_E3: Boolean;
  bFind1: Boolean;
  bFind2: Boolean;
  hFindData1: THandle;
  pBuff1: PAnsiChar;
  File1: TSearchRec;
  FileStream1: TFileStream;
  MemoryStream1 :TMemoryStream;
begin
  //mF
  if StrLoadFileName = '' then
  begin
    showmessage('t@Cǂݍ܂Ă܂');
    Exit;
  end;
  //t@C擾
  hFindData1 := FindFirst(StrLoadFileName, faAnyFile, File1);
  try
    ;
  finally
    FindClose(File1);
  end;
  //s
  if hFindData1 <> 0 then
  begin
    ShowMessage('t@C܂P');
    Exit;
  end;

    //ă`FbNTCY̏w
    iMax1 := File1.Size;
    if iMax1 > CHAR_UNKNOW_CHECK_SIZE_MAX then
    begin
      iMax1 := CHAR_UNKNOW_CHECK_SIZE_MAX;
    end;

    //
    CharCode := ENCODE_SJIS;
    RetCode := RETURN_CRLF;
    BoolUnKnownCharCode := FALSE;
    //TCY
    if File1.Size = 0 then
    begin
      CharCode := ENCODE_SJIS;
      RetCode := RETURN_NONE;
    end
    //eL
    else
    begin
      //TCYw肵ăm
      pBuff1 := AllocMem(File1.Size);
      try
        //ǂݍ
        try
          //̓t@Cݒ肵ēǂݍ
          FileStream1 := TFileStream.Create(FileName, fmOpenRead);
          try
            FileStream1.Read(pBuff1^, File1.Size);
          finally
            FileStream1.Free;
          end;
        except
          //ReadOnly J 103 ̃G[ɂȂ΍
        end;

        //

        //Size = 1
        if      File1.Size = 1 then
        begin
          CharCode := ENCODE_SJIS;
        end
        //Size == 2
        else if File1.Size = 2 then
        begin
          //ݒ
          i := 0;
          //UTF16 Little BOM
          if      (pBuff1[i+0] = #$FF) and (pBuff1[i+1] = #$FE) then
          begin
            CharCode := ENCODE_UTF16_BOM_ON_LE;
          end
          //UTF16 Big BOM
          else if (pBuff1[i+0] = #$FE) and (pBuff1[i+1] = #$FF) then
          begin
            CharCode := ENCODE_UTF16_BOM_ON_BE;
          end

          // EUC Ђ炪
          else if ( ((pBuff1[i+0] = #$A4) and (pBuff1[i+1] >= #$A1)) and
                    ((pBuff1[i+0] = #$A4) and (pBuff1[i+1] <= #$F3)) )
          then
          begin
            CharCode := ENCODE_EUCJP;
          end
          // EUC J^Ji
          else if ( ((pBuff1[i+0] = #$A5) and (pBuff1[i+1] >= #$A1)) and
                    ((pBuff1[i+0] = #$A5) and (pBuff1[i+1] <= #$F6)) )
          then
          begin
            CharCode := ENCODE_EUCJP;
          end
          // EUC 
          else if ( ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] >= #$B0)) and
                    ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] <= #$B9)) )
          then
          begin
            CharCode := ENCODE_EUCJP;
          end
          // EUC p
          else if ( ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] >= #$C1)) and
                    ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] <= #$Da)) )
          then
          begin
            CharCode := ENCODE_EUCJP;
          end
          // EUC p
          else if ( ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] >= #$E1)) and
                    ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] <= #$FA)) )
          then
          begin
            CharCode := ENCODE_EUCJP;
          end

          // SJIS Ђ炪
          else if ( ((pBuff1[i+0] = #$82) and (pBuff1[i+1] >= #$9F)) and
                    ((pBuff1[i+0] = #$82) and (pBuff1[i+1] <= #$F1)) )
          then
          begin
            CharCode := ENCODE_SJIS;
          end
          // SJIS J^Ji
          else if ( ((pBuff1[i+0] = #$83) and (pBuff1[i+1] >= #$40)) and
                    ((pBuff1[i+0] = #$83) and (pBuff1[i+1] <= #$96)) )
          then
          begin
             CharCode := ENCODE_SJIS;
          end
          // SJIS 
          else if ( ((pBuff1[i+0] = #$82) and (pBuff1[i+1] >= #$4F)) and
                    ((pBuff1[i+0] = #$82) and (pBuff1[i+1] <= #$58)) )
          then
          begin
             CharCode := ENCODE_SJIS;
          end
          // SJIS p
          else if ( ((pBuff1[i+0] = #$82) and (pBuff1[i+1] >= #$60)) and
                    ((pBuff1[i+0] = #$82) and (pBuff1[i+1] <= #$79)) )
          then
          begin
             CharCode := ENCODE_SJIS;
          end
          // SJIS p
          else if ( ((pBuff1[i+0] = #$82) and (pBuff1[i+1] >= #$81)) and
                    ((pBuff1[i+0] = #$82) and (pBuff1[i+1] <= #$9A)) )
          then
          begin
            CharCode := ENCODE_SJIS;
          end

          // JIS Ђ炪
          else if ( ((pBuff1[i+0] = #$24) and (pBuff1[i+1] >= #$21)) and
                    ((pBuff1[i+0] = #$24) and (pBuff1[i+1] <= #$73)) )
          then
          begin
            CharCode := ENCODE_JIS;
          end
          // JIS J^Ji
          else if ( ((pBuff1[i+0] = #$25) and (pBuff1[i+1] >= #$21)) and
                    ((pBuff1[i+0] = #$25) and (pBuff1[i+1] <= #$76)) )
          then
          begin
            CharCode := ENCODE_JIS;
          end
          // JIS 
          else if ( ((pBuff1[i+0] = #$23) and (pBuff1[i+1] >= #$30)) and
                    ((pBuff1[i+0] = #$23) and (pBuff1[i+1] <= #$39)) )
          then
          begin
            CharCode := ENCODE_JIS;
          end
          // JIS p
          else if ( ((pBuff1[i+0] = #$23) and (pBuff1[i+1] >= #$41)) and
                    ((pBuff1[i+0] = #$23) and (pBuff1[i+1] <= #$5A)) )
          then
          begin
            CharCode := ENCODE_JIS;
          end
          // JIS p
          else if ( ((pBuff1[i+0] = #$23) and (pBuff1[i+1] >= #$61)) and
                    ((pBuff1[i+0] = #$23) and (pBuff1[i+1] <= #$7A)) )
          then
          begin
            CharCode := ENCODE_JIS;
          end

          //
          else
          begin
            CharCode := ENCODE_SJIS;
          end;
        end
        //Size >= 3
        else
        begin
          //UTF8 BOM
          if (pBuff1[0] = #$EF) and (pBuff1[1] = #$BB) and (pBuff1[2] = #$BF) then
          begin
            CharCode := ENCODE_UTF8_BOM_ON;
          end
          //UTF16 Little BOM
          else if(pBuff1[0] = #$FF) and (pBuff1[1] = #$FE) then
          begin
            CharCode := ENCODE_UTF16_BOM_ON_LE;
          end
          //UTF16 Big BOM
          else if (pBuff1[0] = #$FE) and (pBuff1[1] = #$FF) then
          begin
            CharCode := ENCODE_UTF16_BOM_ON_BE;
          end
          //ȊO
          else
          begin
            //
            bFind1   := FALSE;
            bSJIS_E3 := FALSE;

            //
            if bFind1 = FALSE then
            begin
              // UTF8  SJIS ͓͈͂̂ōŏ UTF8 𔻒肷
              bSJIS_E3 := FALSE;
              for i := 0 to (File1.Size-4) do
              begin
                //͈͂
                if i >= 1 then
                begin
                  //UTF8  #$E3  SJIS ̊Y͈͂ɂꍇ͏
                  if( ((pBuff1[i-1] = #$81) and (pBuff1[i-0] >= #$E3)) or
                      ((pBuff1[i-1] = #$82) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$83) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$84) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$85) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$86) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$87) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$88) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$89) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$8A) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$8B) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$8C) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$8D) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$8E) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$8F) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$90) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$91) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$92) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$93) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$94) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$95) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$96) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$97) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$98) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$99) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$9A) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$9B) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$9C) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$9D) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$9E) and (pBuff1[i-0] <= #$E3)) or
                      ((pBuff1[i-1] = #$9F) and (pBuff1[i-0] <= #$E3))
                      )
                  then
                  begin
                    bSJIS_E3 := TRUE;
                  end
                  else
                  begin
                    bSJIS_E3 := FALSE;
                  end;
                end;

                // UTF8 Ђ炪
                if( ((pBuff1[i+0] = #$E3) and (pBuff1[i+1] = #$81) and (pBuff1[i+2] >= #$81)) and
                    ((pBuff1[i+0] = #$E3) and (pBuff1[i+1] = #$81) and (pBuff1[i+2] <= #$BF)) )
                then
                begin
                  if bSJIS_E3 = FALSE then
                  begin
                    CharCode := ENCODE_UTF8_BOM_OFF;
                    bFind1 := TRUE;
                    Break;
                  end;
                end;
                // UTF8 Ђ炪
                if( ((pBuff1[i+0] = #$E3) and (pBuff1[i+1] = #$82) and (pBuff1[i+2] >= #$80)) and
                    ((pBuff1[i+0] = #$E3) and (pBuff1[i+1] = #$82) and (pBuff1[i+2] <= #$93)) )
                then
                begin
                  if bSJIS_E3 = FALSE then
                  begin
                    CharCode := ENCODE_UTF8_BOM_OFF;
                    bFind1 := TRUE;
                    Break;
                  end;
                end;
                // UTF8 J^Ji
                if ( ((pBuff1[i+0] = #$E3) and (pBuff1[i+1] = #$82) and (pBuff1[i+2] >= #$A1)) and
                     ((pBuff1[i+0] = #$E3) and (pBuff1[i+1] = #$82) and (pBuff1[i+2] <= #$BF)) )
                then
                begin
                  if bSJIS_E3 = FALSE then
                  begin
                    CharCode := ENCODE_UTF8_BOM_OFF;
                    bFind1 := TRUE;
                    Break;
                  end;
                end;
                // UTF8 J^Ji
                if ( ((pBuff1[i+0] = #$E3) and (pBuff1[i+1] = #$83) and (pBuff1[i+2] >= #$80)) and
                     ((pBuff1[i+0] = #$E3) and (pBuff1[i+1] = #$83) and (pBuff1[i+2] <= #$B6)) )
                then
                begin
                  if bSJIS_E3 = FALSE then
                  begin
                    CharCode := ENCODE_UTF8_BOM_OFF;
                    bFind1 := TRUE;
                    Break;
                  end;
                end;
                // UTF8 
                if ( ((pBuff1[i+0] = #$EF) and (pBuff1[i+1] = #$BC) and (pBuff1[i+2] >= #$90)) and
                     ((pBuff1[i+0] = #$EF) and (pBuff1[i+1] = #$BC) and (pBuff1[i+2] <= #$99)) )
                then
                begin
                  if bSJIS_E3 = FALSE then
                  begin
                    CharCode := ENCODE_UTF8_BOM_OFF;
                    bFind1 := TRUE;
                    Break;
                  end;
                end;
                // UTF8 p
                if ( ((pBuff1[i+0] = #$EF) and (pBuff1[i+1] = #$BC) and (pBuff1[i+2] >= #$A1)) and
                     ((pBuff1[i+0] = #$EF) and (pBuff1[i+1] = #$BC) and (pBuff1[i+2] <= #$BA)) )
                then
                begin
                  if bSJIS_E3 = FALSE then
                  begin
                    CharCode := ENCODE_UTF8_BOM_OFF;
                    bFind1 := TRUE;
                    Break;
                  end;
                end;
                // UTF8 p
                if ( ((pBuff1[i+0] = #$EF) and (pBuff1[i+1] = #$BD) and (pBuff1[i+2] >= #$81)) and
                     ((pBuff1[i+0] = #$EF) and (pBuff1[i+1] = #$BD) and (pBuff1[i+2] <= #$9A)) )
                then
                begin
                  if bSJIS_E3 = FALSE then
                  begin
                    CharCode := ENCODE_UTF8_BOM_OFF;
                    bFind1 := TRUE;
                    Break;
                  end;
                end;
              end;
            end;

            //
            if bFind1 = FALSE then
            begin
              for i := 0 to (File1.Size-3) do
              begin
                // SJIS Ђ炪
                if ( ((pBuff1[i+0] = #$82) and (pBuff1[i+1] >= #$9F)) and
                     ((pBuff1[i+0] = #$82) and (pBuff1[i+1] <= #$F1)) )
                then
                begin
                  CharCode := ENCODE_SJIS;
                  bFind1 := TRUE;
                  Break;
                end;
                // SJIS J^Ji
                if ( ((pBuff1[i+0] = #$83) and (pBuff1[i+1] >= #$40)) and
                     ((pBuff1[i+0] = #$83) and (pBuff1[i+1] <= #$96)) )
                then
                begin
                  CharCode := ENCODE_SJIS;
                  bFind1 := TRUE;
                  Break;
                end;
                // SJIS 
                if ( ((pBuff1[i+0] = #$82) and (pBuff1[i+1] >= #$4F)) and
                     ((pBuff1[i+0] = #$82) and (pBuff1[i+1] <= #$58)) )
                then
                begin
                  CharCode := ENCODE_SJIS;
                  bFind1 := TRUE;
                  Break;
                end;
                // SJIS p
                if ( ((pBuff1[i+0] = #$82) and (pBuff1[i+1] >= #$60)) and
                     ((pBuff1[i+0] = #$82) and (pBuff1[i+1] <= #$79)) )
                then
                begin
                  CharCode := ENCODE_SJIS;
                  bFind1 := TRUE;
                  Break;
                end;
                // SJIS p
                if ( ((pBuff1[i+0] = #$82) and (pBuff1[i+1] >= #$81)) and
                     ((pBuff1[i+0] = #$82) and (pBuff1[i+1] <= #$9A)) )
                then
                begin
                  CharCode := ENCODE_SJIS;
                  bFind1 := TRUE;
                  Break;
                end;
              end;
            end;


            //
            if bFind1 = FALSE then
            begin
              for i := 0 to (File1.Size-3) do
              begin
                // EUC Ђ炪
                if ( ((pBuff1[i+0] = #$A4) and (pBuff1[i+1] >= #$A1)) and
                     ((pBuff1[i+0] = #$A4) and (pBuff1[i+1] <= #$F3)) )
                then
                begin
                  CharCode := ENCODE_EUCJP;
                  bFind1 := TRUE;
                  Break;
                end;
                // EUC J^Ji
                if ( ((pBuff1[i+0] = #$A5) and (pBuff1[i+1] >= #$A1)) and
                     ((pBuff1[i+0] = #$A5) and (pBuff1[i+1] <= #$F6)) )
                then
                begin
                  CharCode := ENCODE_EUCJP;
                  bFind1 := TRUE;
                  Break;
                end;
                // EUC 
                if ( ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] >= #$B0)) and
                     ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] <= #$B9)) )
                then
                begin
                  CharCode := ENCODE_EUCJP;
                  bFind1 := TRUE;
                  Break;
                end;
                // EUC p
                if ( ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] >= #$C1)) and
                     ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] <= #$Da)) )
                then
                begin
                  CharCode := ENCODE_EUCJP;
                  bFind1 := TRUE;
                  Break;
                end;
                // EUC p
                if ( ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] >= #$E1)) and
                     ((pBuff1[i+0] = #$A3) and (pBuff1[i+1] <= #$FA)) )
                then
                begin
                  CharCode := ENCODE_EUCJP;
                  bFind1 := TRUE;
                  Break;
                end;
              end;
            end;


            //
            if bFind1 = FALSE then
            begin
              for i := 0 to (File1.Size-3) do
              begin
                // JIS Ђ炪
                if ( ((pBuff1[i+0] = #$24) and (pBuff1[i+1] >= #$21)) and
                     ((pBuff1[i+0] = #$24) and (pBuff1[i+1] <= #$73)) )
                then
                begin
                  CharCode := ENCODE_JIS;
                  bFind1 := TRUE;
                  Break;
                end;
                // JIS J^Ji
                if ( ((pBuff1[i+0] = #$25) and (pBuff1[i+1] >= #$21)) and
                     ((pBuff1[i+0] = #$25) and (pBuff1[i+1] <= #$76)) )
                then
                begin
                  CharCode := ENCODE_JIS;
                  bFind1 := TRUE;
                  Break;
                end;
                // JIS 
                if ( ((pBuff1[i+0] = #$23) and (pBuff1[i+1] >= #$30)) and
                     ((pBuff1[i+0] = #$23) and (pBuff1[i+1] <= #$39)) )
                then
                begin
                  CharCode := ENCODE_JIS;
                  bFind1 := TRUE;
                  Break;
                end;
                // JIS p
                if ( ((pBuff1[i+0] = #$23) and (pBuff1[i+1] >= #$41)) and
                     ((pBuff1[i+0] = #$23) and (pBuff1[i+1] <= #$5A)) )
                then
                begin
                  CharCode := ENCODE_JIS;
                  bFind1 := TRUE;
                  Break;
                end;
                // JIS p
                if ( ((pBuff1[i+0] = #$23) and (pBuff1[i+1] >= #$61)) and
                     ((pBuff1[i+0] = #$23) and (pBuff1[i+1] <= #$7A)) )
                then
                begin
                  CharCode := ENCODE_JIS;
                  bFind1 := TRUE;
                  Break;
                end;
              end;
            end;


            //xoȂ
            if bFind1 = FALSE then
            begin
              //
            end;
            //xoȂ
            if bSJIS_E3 = FALSE then
            begin
              //
            end;
          end;
        end;

        //s
        bFind2 := FALSE;
        //UTF16  BOM ɂJnʒuύX
        if (CharCode = ENCODE_UTF16_BOM_ON_BE) or (CharCode = ENCODE_UTF16_BOM_ON_LE)  then
        begin
          i := 2;
        end
        else
        begin
          i := 0;
        end;
        //2ɏ̂ for ͎gȂ
        while i <= (File1.Size-4) do
        begin
          // UTF16LE CRLF
          if      ( (pBuff1[i+0] = #13) and (pBuff1[i+1] = #00) and (pBuff1[i+2] = #10) and (pBuff1[i+3] = #00) )
          then
          begin
            RetCode := RETURN_CRLF;
            bFind2 := TRUE;
            //R[h̔ł͖Ȃ̂ŉs̕тŔ
            if (CharCode <> ENCODE_UTF16_BOM_ON_BE) and (CharCode <> ENCODE_UTF16_BOM_ON_LE)then
            begin
              CharCode := ENCODE_UTF16_BOM_OFF_LE;
            end;
            Break;
          end
          // UTF16BE CRLF
          else if ( (pBuff1[i+0] = #00) and (pBuff1[i+1] = #13) and (pBuff1[i+2] = #00) and (pBuff1[i+3] = #10) )
          then
          begin
            RetCode := RETURN_CRLF;
            bFind2 := TRUE;
            //R[h̔ł͖Ȃ̂ŉs̕тŔ
            if (CharCode <> ENCODE_UTF16_BOM_ON_BE) and (CharCode <> ENCODE_UTF16_BOM_ON_LE)then
            begin
              CharCode := ENCODE_UTF16_BOM_OFF_BE;
            end;
            Break;
          end
          //
          else
          begin
          end;
          //2̏
          i := i+2;
        end;

        //
        if bFind2 = FALSE then
        begin
          //2ɏ̂ for ͎gȂ
          i := 0;
          while i <= (File1.Size-2) do
          begin
            // UTF16BE CR
            if      ( (pBuff1[i+0] = #00) and (pBuff1[i+1] = #13) )
            then
            begin
              RetCode := RETURN_CR;
              bFind2 := TRUE;
              //R[h̔ł͖Ȃ̂ŉs̕тŔ
              if (CharCode <> ENCODE_UTF16_BOM_ON_BE) and (CharCode <> ENCODE_UTF16_BOM_ON_LE)then
              begin
                CharCode := ENCODE_UTF16_BOM_OFF_BE;
              end;
              Break;
            end
            // UTF16LE CR
            else if ( (pBuff1[i+0] = #13) and (pBuff1[i+1] = #00) )
            then
            begin
              RetCode := RETURN_CR;
              bFind2 := TRUE;
              //R[h̔ł͖Ȃ̂ŉs̕тŔ
              if (CharCode <> ENCODE_UTF16_BOM_ON_BE) and (CharCode <> ENCODE_UTF16_BOM_ON_LE)then
              begin
                CharCode := ENCODE_UTF16_BOM_OFF_LE;
              end;
              Break;
            end
            // UTF16BE LF
            else if ( (pBuff1[i+0] = #00) and (pBuff1[i+1] = #10) )
            then
            begin
              RetCode := RETURN_LF;
              bFind2 := TRUE;
              //R[h̔ł͖Ȃ̂ŉs̕тŔ
              if (CharCode <> ENCODE_UTF16_BOM_ON_BE) and (CharCode <> ENCODE_UTF16_BOM_ON_LE)then
              begin
                CharCode := ENCODE_UTF16_BOM_OFF_BE;
              end;
              Break;
            end
            // UTF16LE LF
            else if ( (pBuff1[i+0] = #10) and (pBuff1[i+1] = #00) )
            then
            begin
              RetCode := RETURN_LF;
              bFind2 := TRUE;
              //R[h̔ł͖Ȃ̂ŉs̕тŔ
              if (CharCode <> ENCODE_UTF16_BOM_ON_BE) and (CharCode <> ENCODE_UTF16_BOM_ON_LE)then
              begin
                CharCode := ENCODE_UTF16_BOM_OFF_LE;
              end;
              Break;
            end
            //
            else
            begin
            end;
            //2̏
            i := i+2;
          end;
        end;

        //
        if bFind2 = FALSE then
        begin
          iCr1 := 0;
          iLf1 := 0;
          BoolLfInLine := FALSE;
          for i := 0 to (File1.Size-1) do
          begin
            // CR
            if      ( (pBuff1[i+0] = #13) )
            then
            begin
              iCr1 := iCr1+1;
            end
            // LF
            else if ( (pBuff1[i+0] = #10) )
            then
            begin
              iLf1 := iLf1+1;
            end
            //
            else
            begin
            end;
          end;
          //ݒ
          if (iCr1 > 0) and (iLf1 > 0) then
          begin
            RetCode := RETURN_CRLF;
            //s Lf 
            if iCr1 <> iLf1 then
            begin
              BoolLfInLine := TRUE;
            end
            else
            begin
              BoolLfInLine := FALSE;
            end;
          end
          else if (iCr1 > 0) then
          begin
              RetCode := RETURN_CR;
          end
          else if (iLf1 > 0) then
          begin
              RetCode := RETURN_LF;
          end
          else
          begin
              RetCode := RETURN_NONE;
          end;
        end;
        //x\Ȃ
        if bFind2 = TRUE then
        begin
        end;

        //ēx
         if (CharCode = ENCODE_UTF16_BOM_ON_BE) or (CharCode = ENCODE_UTF16_BOM_OFF_BE) or
            (CharCode = ENCODE_UTF16_BOM_ON_LE) or (CharCode = ENCODE_UTF16_BOM_OFF_LE) then
         begin
           //
         end
         //ȊO
         else
         begin
           //m
           MemoryStream1 := TMemoryStream.Create;
           try
              //
              for i := 0 to (iMax1-1) do
              begin
                //s
                //#$1B  JIS Ŏgp
                if (pBuff1[i] in [#$00..#$08, #$0B..#$0C, #$0E..#$1A, #$1C..#$1F]) then
                begin
                  BoolUnKnownCharCode := TRUE;
                  Break;
                end;
              end;
              //s̏ꍇ
              if BoolUnKnownCharCode = TRUE then
              begin
                for i := 0 to (File1.Size-1) do
                begin
                  if (pBuff1[i] in [#$00..#$08, #$0B..#$0C, #$0E..#$1F]) then
                  begin
                    //
                  end
                  else
                  begin
                    MemoryStream1.Write(pBuff1[i], Length(pBuff1[i]));
                  end;
                end;
              end;
              //ۑ
              sStr2 := ExtractFilePath(ParamStr(0)) + DLL_FOLDER_NAME + '\' + CONVERT_FOLDER_NAME;
              sStr3 := sStr2 + '\' + UNKOWN_DST_LIST;
              MemoryStream1.SaveToFile(sStr3);
           finally
             //j
             MemoryStream1.Free;
           end;
         end;
      finally
        FreeMem(pBuff1);
      end;
    end;


end;



procedure TFormCsvText.LoadFile(Sender: TObject);
var
  hFindData1: THandle;
  File1: TSearchRec;
  Enc1: TEncoding;
begin
  //mF
  if StrLoadFileName = '' then
  begin
    showmessage('t@Cǂݍ܂Ă܂');
    Exit;
  end;
  //t@C擾
  hFindData1 := FindFirst(StrLoadFileName, faAnyFile, File1);
  try
    ;
  finally
    FindClose(File1);
  end;
  //s
  if hFindData1 <> 0 then
  begin
    ShowMessage('t@C܂P');
    Exit;
  end;
  //t@CTCY
  IntLoadFileSize := File1.Size;

  //\
  StatusBar1.Panels.Items[3].Text := '';
  StatusBar1.Panels.Items[4].Text := '';
  StatusBar1.Panels.Items[5].Text := '';
  StatusBar1.Panels.Items[6].Text := '';

  //
  IntReadEncode1 := ENCODE_UNKNOWN;
  IntReadReturn1 := RETURN_NONE;

  //͎o
  if MenuReadAuto.Checked = TRUE then
  begin
    //擾
    GetCharCodeRetCode(StrLoadFileName, IntReadEncode1, IntReadReturn1, Sender);
  end
  //͐ݒɏ]
  else
  begin
    //擾 spɎs
    GetCharCodeRetCode(StrLoadFileName, IntReadEncode1, IntReadReturn1, Sender);
    //ݒ
    if      MenuReadSJIS.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_SJIS;
    end
    else if MenuReadJIS.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_JIS;
    end
    else if MenuReadEUCJP.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_EUCJP;
    end
    else if MenuReadUTF7.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF7;
    end
    else if MenuReadUTF8Bom.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF8_BOM_ON;
    end
    else if MenuReadUTF8.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF8_BOM_OFF;
    end
    else if MenuReadUTF16BomBe.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF16_BOM_ON_BE;
    end
    else if MenuReadUTF16Be.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF16_BOM_OFF_BE;
    end
    else if MenuReadUTF16BomLe.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF16_BOM_ON_LE;
    end
    else if MenuReadUTF16Le.Checked = TRUE then
    begin
      IntReadEncode1 := ENCODE_UTF16_BOM_OFF_LE;
    end
    else
    begin
      IntReadEncode1 := ENCODE_UNKNOWN;
    end;
  end;

  //sɉs
  if BoolLfInLine = TRUE then
  begin
    //\
    showmessage('s(CrLf)܂łɁAɉs(Lf)܂');
  end;


  //UnKnown
  if BoolUnKnownCharCode = TRUE then
  begin
    //\
    showmessage('sȕ܂ł܂̂' + #13#10 + '֎~t@CƂĈ܂');
    //f
    StrLoadChar := CHAR_BINARY;
    StrSaveChar := CHAR_BINARY;
    //Ǎ
    Memo1.Lines.LoadFromFile(ExtractFilePath(ParamStr(0)) + UNKOWN_DST_LIST);
    StrListLoadFileTmp.LoadFromFile(ExtractFilePath(ParamStr(0)) + UNKOWN_DST_LIST);
  end

  //SJIS
  else if IntReadEncode1 = ENCODE_SJIS then
  begin
    //f
    StrLoadChar := CHAR_SJIS;
    StrSaveChar := CHAR_SJIS;
    //Ǎ
    Enc1 := TEncoding.GetEncoding(932);
    try
      //Ǎ
      Memo1.Lines.LoadFromFile(StrLoadFileName, Enc1);
      StrListLoadFileTmp.LoadFromFile(StrLoadFileName, Enc1);
    finally
      Enc1.Free;
    end;
  end

  //JIS
  else if IntReadEncode1 = ENCODE_JIS then
  begin
    //f
    StrLoadChar := CHAR_JIS;
    StrSaveChar := CHAR_JIS;
    //Ǎ
    Enc1 := TEncoding.GetEncoding(50220);
    try
      //Ǎ
      Memo1.Lines.LoadFromFile(StrLoadFileName, Enc1);
      StrListLoadFileTmp.LoadFromFile(StrLoadFileName, Enc1);
    finally
      Enc1.Free;
    end;
  end

  //EUC
  else if IntReadEncode1 = ENCODE_EUCJP then
  begin
    //f
    StrLoadChar := CHAR_EUCJP;
    StrSaveChar := CHAR_EUCJP;
    //Ǎ
    Enc1 := TEncoding.GetEncoding(20932);
    try
      //Ǎ
      Memo1.Lines.LoadFromFile(StrLoadFileName, Enc1);
      StrListLoadFileTmp.LoadFromFile(StrLoadFileName, Enc1);
    finally
      Enc1.Free;
    end;
  end

  //UTF7
  else if IntReadEncode1 = ENCODE_UTF7 then
  begin
    //f
    StrLoadChar := CHAR_UTF7;
    StrSaveChar := CHAR_UTF7;
    //Ǎ
    Memo1.Lines.LoadFromFile(StrLoadFileName, TEncoding.UTF7);
    StrListLoadFileTmp.LoadFromFile(StrLoadFileName, TEncoding.UTF7);
  end

  //UTF8 BOM
  else if IntReadEncode1 = ENCODE_UTF8_BOM_ON then
  begin
    //f
    StrLoadChar := CHAR_UTF8_BOM_ON;
    StrSaveChar := CHAR_UTF8_BOM_ON;
    //Ǎ
    Memo1.Lines.LoadFromFile(StrLoadFileName);
    StrListLoadFileTmp.LoadFromFile(StrLoadFileName);
   end

  //UTF8
  else if IntReadEncode1 = ENCODE_UTF8_BOM_OFF then
  begin
    //f
    StrLoadChar := CHAR_UTF8_BOM_OFF;
    StrSaveChar := CHAR_UTF8_BOM_OFF;
    //Ǎ
    Memo1.Lines.LoadFromFile(StrLoadFileName, TEncoding.UTF8);
    StrListLoadFileTmp.LoadFromFile(StrLoadFileName, TEncoding.UTF8);
   end

  //UTF16 BOM BE
  else if IntReadEncode1 = ENCODE_UTF16_BOM_ON_BE then
  begin
    //f
    StrLoadChar := CHAR_UTF16_BOM_ON_BE;
    StrSaveChar := CHAR_UTF16_BOM_ON_BE;
    //Ǎ
    Memo1.Lines.LoadFromFile(StrLoadFileName);
    StrListLoadFileTmp.LoadFromFile(StrLoadFileName);
   end

  //UTF16 BE
  else if IntReadEncode1 = ENCODE_UTF16_BOM_OFF_BE then
  begin
    //f
    StrLoadChar := CHAR_UTF16_BOM_OFF_BE;
    StrSaveChar := CHAR_UTF16_BOM_OFF_BE;
    //Ǎ
    Memo1.Lines.LoadFromFile(StrLoadFileName, TEncoding.BigEndianUnicode);
    StrListLoadFileTmp.LoadFromFile(StrLoadFileName, TEncoding.BigEndianUnicode);
   end

  //UTF16 BOM LE
  else if IntReadEncode1 = ENCODE_UTF16_BOM_ON_LE then
  begin
    //f
    StrLoadChar := CHAR_UTF16_BOM_ON_LE;
    StrSaveChar := CHAR_UTF16_BOM_ON_LE;
    //Ǎ
    Memo1.Lines.LoadFromFile(StrLoadFileName);
    StrListLoadFileTmp.LoadFromFile(StrLoadFileName);
   end

  //UTF16 LE
  else if IntReadEncode1 = ENCODE_UTF16_BOM_OFF_LE then
  begin
    StrLoadChar := CHAR_UTF16_BOM_OFF_LE;
    StrSaveChar := CHAR_UTF16_BOM_OFF_LE;
    //Ǎ
    Memo1.Lines.LoadFromFile(StrLoadFileName, TEncoding.Unicode);
    StrListLoadFileTmp.LoadFromFile(StrLoadFileName, TEncoding.Unicode);
   end

  //͂Ȃ
  else
  begin
    //SJIS
    StrLoadChar := CHAR_SJIS;
    StrSaveChar := CHAR_SJIS;
  end;

  //f
  StrListTextEx.Assign(StrListLoadFileTmp);

  //
  Memo1.SelStart := 0;

  //s
  if IntReadReturn1 = RETURN_CRLF then
  begin
    //CRLF
    StrLoadReturn := RET_CRLF;
    StrSaveReturn := RET_CRLF;
  end
  else if IntReadReturn1 = RETURN_CR then
  begin
    //CR
    StrLoadReturn := RET_CR;
  end
  else if IntReadReturn1 = RETURN_LF then
  begin
    //Lf
    StrLoadReturn := RET_LF;
    StrSaveReturn := RET_LF;
  end
  else if IntReadReturn1 = RETURN_NONE then
  begin
    //
    StrLoadReturn := RET_NONE;
    StrSaveReturn := RET_NONE;
  end
  else
  begin
    //CRLF
    StrLoadReturn := RET_CRLF;
    StrSaveReturn := RET_CRLF;
  end;


  //֎~ ܂ s
  if (Bool(File1.Attr and FILE_ATTRIBUTE_READONLY) = TRUE) or (BoolUnKnownCharCode = TRUE) then
  begin
    //ݒ
    Memo1.ReadOnly := TRUE;
    Memo1.Modified := FALSE;
    BoolFileReadOnly := TRUE;
    //\
    StatusBar1.Panels.Items[3].Text := FILE_READ_ONLY;
    StatusBar1.Panels.Items[4].Text := StrLoadChar;
    StatusBar1.Panels.Items[5].Text := StrLoadReturn;
    StatusBar1.Panels.Items[6].Text := '';
  end
  else
  begin
    //ݒ
    Memo1.ReadOnly := TRUE;
    Memo1.Modified := FALSE;
    BoolFileReadOnly := FALSE;
    //\
    StatusBar1.Panels.Items[3].Text := '';
    StatusBar1.Panels.Items[4].Text := StrLoadChar;
    StatusBar1.Panels.Items[5].Text := StrLoadReturn;
    StatusBar1.Panels.Items[6].Text := '';
  end;

  //ݒ
  StrLoadCharMem   := StrLoadChar;
  StrLoadReturnMem := StrLoadReturn;
  StrSaveCharMem   := StrSaveChar;
  StrSaveReturnMem := StrSaveReturn;


  //f
  if MenuTab2.Checked = TRUE then
  begin
    TabStops[0] := 4*2;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
    Memo1.Invalidate;
  end;
  if MenuTab4.Checked = TRUE then
  begin
    TabStops[0] := 4*4;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
    Memo1.Invalidate;
  end;
  if MenuTab6.Checked = TRUE then
  begin
    TabStops[0] := 4*6;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
    Memo1.Invalidate;
  end;
  if MenuTab8.Checked = TRUE then
  begin
    TabStops[0] := 4*8;
    Memo1.Perform(EM_SETTABSTOPS, 1, Longint(@TabStops));
    Memo1.Invalidate;
  end;

  //\XV
  StatusBar1.Panels.Items[2].Text := Format('%d', [Memo1.Lines.Count]) + 's';

  //sԍݒ
  SetLineNumber(Sender);
  if MenuShowLineNumber.Checked = TRUE then
  begin
    Memo9.Visible  := TRUE;
    Timer6.Enabled := TRUE;
  end
  else
  begin
    Memo9.Visible  := FALSE;
    Timer6.Enabled := FALSE;
  end;
end;

procedure TFormCsvText.LoadTabSeparateFile(Sender: TObject);
var
  sStr4: String;
  iRow1: Integer;
  iRow4: Integer;
  iLen1: Integer;
  iStart1: Integer;
  iEnd1: Integer;
  StrList1: TStringList;
  StrStream1: TStringStream;
  pWideChar1: PWideChar;
  hFindData1: THandle;
  File1: TSearchRec;
begin
  //mF
  if StrLoadFileName = '' then
  begin
    showmessage('t@Cǂݍ܂Ă܂');
    Exit;
  end;
  //t@C擾
  hFindData1 := FindFirst(StrLoadFileName, faAnyFile, File1);
  try
    ;
  finally
    FindClose(File1);
  end;
  //s
  if hFindData1 <> 0 then
  begin
    ShowMessage('t@C܂P');
    Exit;
  end;

  //Ǎ
  LoadCsvFile(Sender);

  //m
  StrList1 := TStringList.Create;

  //Jnʒu
  iStart1 := 0;
  //ŏIʒu
  iEnd1 := StrListLoadCsvFile.Count;

  //sƃ^uu
  for iRow1 := iStart1 to iEnd1-1 do
  begin
    //]
    pWideChar1 := PWideChar(StrListLoadCsvFile[iRow1]);
    //
    iLen1 := Length(StrListLoadCsvFile[iRow1]);
    //
    sStr4 := '';
    //]
    for iRow4 := 0 to iLen1-1 do
    begin
      //s
      if      pWideChar1[iRow4] = #10 then
      begin
        sStr4 := sStr4 + StrReturnMark;
      end
      //^u
      else if pWideChar1[iRow4] = #9 then
      begin
        sStr4 := sStr4 + StrCsvSeparater;
      end
      //ȊO
      else
      begin
        sStr4 := sStr4 + pWideChar1[iRow4];
      end;
    end;
    //ݒ
    StrList1.Add(sStr4);
  end;


  //t@C܂̓Xg[͍ɕ\\
  StrStream1 := TStringStream.Create(StrList1.Text);
  Memo1.Lines.BeginUpdate;
  Memo1.Lines.LoadFromStream(StrStream1);
  Memo1.Lines.EndUpdate;
  StrStream1.Free;

  //j
  StrList1.Free;

  //\XV
  StatusBar1.Panels.Items[2].Text := Format('%d', [Memo1.Lines.Count]) + 's';

  //sԍݒ
  SetLineNumber(Sender);
  if MenuShowLineNumber.Checked = TRUE then
  begin
    Memo9.Visible  := TRUE;
    Timer6.Enabled := TRUE;
  end
  else
  begin
    Memo9.Visible  := FALSE;
    Timer6.Enabled := FALSE;
  end;
end;

procedure TFormCsvText.SetCsvFile(Sender: TObject);
var
  sStr1: String;
  sStr2: String;
  sStr3: String;
  sStr4: String;
  sStr5: String;
  sStr6: String;
  iRow1: Integer;
  iRow2: Integer;
  iRow4: Integer;
  iRow5: Integer;
  iLen1: Integer;
  iPos1: Integer;
  iMax2: Integer;
  iMaxCol1: Integer;
  iStart1: Integer;
  iEnd1: Integer;
  CsvList: TStringList;
  StrList1: TStringList;
  StrList2: TStringList;
  StrList3: TStringList;
  StrStream1: TStringStream;
  pWideChar1: PWideChar;
begin
  //m
  StrList1 := TStringList.Create;
  StrList2 := TStringList.Create;
  StrList3 := TStringList.Create;

  //Jnʒu
  iStart1 := 0;
  //ŏIʒu
  iEnd1 := StrListLoadCsvFile.Count;

  //NA
  for iRow1 := 0 to CSVLIST_ITEM_MAX-1 do
  begin
    CsvMax[iRow1] := 0;
  end;

  //su
  for iRow1 := iStart1 to iEnd1-1 do
  begin
    //]
    pWideChar1 := PWideChar(StrListLoadCsvFile[iRow1]);
    //
    iLen1 := Length(StrListLoadCsvFile[iRow1]);
    //
    sStr4 := '';
    //]
    for iRow4 := 0 to iLen1-1 do
    begin
      //s
      if pWideChar1[iRow4] = #10 then
      begin
        sStr4 := sStr4 + StrReturnMark;
      end
      //sȊO
      else
      begin
        sStr4 := sStr4 + pWideChar1[iRow4];
      end;
    end;
    //ݒ
    StrList1.Add(sStr4);
  end;

  //؂u
  for iRow1 := 0 to StrList1.Count-1 do
  begin
    //e
    if StrList1[iRow1] <> '' then
    begin
      //m
      CsvList := TStringList.Create;
      //WJ
      if      (BoolTypeCsvAuto = TRUE) or (BoolTypeCsvAutoFixed = TRUE) then
      begin
        StringToCsvList(StrList1.Strings[iRow1], CsvList);
      end
      else if (BoolTypeCsvComma = TRUE) or (BoolTypeCsvCommaFixed = TRUE) then
      begin
        StringToCsvListEx1(StrList1.Strings[iRow1], CsvList);
      end
      else
      begin
        StringToCsvList(StrList1.Strings[iRow1], CsvList);
      end;
      //쐬
      sStr1 := '';
      for iRow5 := 0 to CsvList.Count-1 do
      begin
        if iRow5 = CsvList.Count-1 then
        begin
          sStr1 := sStr1 + CsvList[iRow5];
        end
        else
        begin
          sStr1 := sStr1 + CsvList[iRow5] + StrCsvSeparater;
        end;
      end;
      //j
      CsvList.Free;
    end
    //e
    else
    begin
      //쐬
      sStr1 := '';
    end;
    //ݒ
    StrList2.Add(sStr1);
  end;


  //ő區ڐƊeڂ̍ő啶擾
  iMaxCol1 := 0;
  for iRow1 := 0 to StrList2.Count-1 do
  begin
    //
    sStr2 := StrList2.Strings[iRow1];
    iMax2 := 0;
    iRow2 := 0;
    while TRUE do
    begin
      //
      iPos1 := Pos(StrCsvSeparater, sStr2);
      if iPos1 > 0 then
      begin
        //擾
        sStr3 := Copy(sStr2, 1, iPos1-1);
        sStr4 := Copy(sStr2, iPos1+1, Length(sStr2));
        sStr2 := sStr4;
        //ǉ
        iMax2 := iMax2+1;
        if iMax2 > iMaxCol1 then
        begin
          iMaxCol1 := iMax2;
        end;

        //eڂ̍ő啶
        if iRow2 < CSVLIST_ITEM_MAX-1 then
        begin
          iLen1 := Length(AnsiString(sStr3));
          if iLen1 > CsvMax[iRow2] then
          begin
            CsvMax[iRow2] := iLen1;
          end;
        end;
        //ǉ
        iRow2 := iRow2+1;
      end
      else
      begin
        //ǉ
        iMax2 := iMax2+1;
        if iMax2 > iMaxCol1 then
        begin
          iMaxCol1 := iMax2;
        end;

        //eڂ̍ő啶
        if iRow2 < CSVLIST_ITEM_MAX-1 then
        begin
          iLen1 := Length(AnsiString(sStr2));
          if iLen1 > CsvMax[iRow2] then
          begin
            CsvMax[iRow2] := iLen1;
          end;
        end;
        //f
        Break;
      end;
    end;
  end;

  //ݒ
  for iRow1 := 0 to StrList2.Count-1 do
  begin
    //
    iRow2 := 0;
    sStr5 := '';
    sStr2 := StrList2.Strings[iRow1];
    while TRUE do
    begin
      //
      iPos1 := Pos(StrCsvSeparater, sStr2);
      if iPos1 > 0 then
      begin
        //擾
        sStr3 := Copy(sStr2, 1, iPos1-1);
        sStr4 := Copy(sStr2, iPos1+1, Length(sStr2));
        sStr2 := sStr4;
        //ݒ
        if      (BoolTypeCsvAuto = TRUE) or (BoolTypeCsvComma = TRUE) then
        begin
            //SĂ̍
            sStr6 := sStr3;
            sStr5 := sStr5 + sStr6 + StrCsvSeparater;
        end
        else if (BoolTypeCsvAutoFixed = TRUE) or (BoolTypeCsvCommaFixed = TRUE) then
        begin
          //ő菬
          if iRow2 < CSVLIST_ITEM_MAX-1 then
          begin
            sStr6 := sStr3 + StringOfChar(' ', CsvMax[iRow2]-Length(AnsiString(sStr3)));
            sStr5 := sStr5 + sStr6 + StrCsvSeparater;
          end
          //ő傫
          else
          begin
            sStr6 := sStr3;
            sStr5 := sStr5 + sStr6 + StrCsvSeparater;
          end;
        end
        else
        begin
            //SĂ̍
            sStr6 := sStr3;
            sStr5 := sStr5 + sStr6 + StrCsvSeparater;
        end;
        //ǉ
        iRow2 := iRow2+1;
      end
      else
      begin
        //ݒ
        if      (BoolTypeCsvAuto = TRUE) or (BoolTypeCsvComma = TRUE) then
        begin
            //؂肪܂͍Ō
            sStr5 :=sStr5 + sStr2;
        end
        else if (BoolTypeCsvAutoFixed = TRUE) or (BoolTypeCsvCommaFixed = TRUE) then
        begin
          //ő菬
          if iRow2 < CSVLIST_ITEM_MAX-1 then
          begin
            sStr6 := sStr2 + StringOfChar(' ', CsvMax[iRow2]-Length(AnsiString(sStr2)));
            sStr5 := sStr5 + sStr6;
          end
          //ő傫
          else
          begin
            //؂肪܂͍Ō
            sStr5 :=sStr5 + sStr2;
          end;
        end
        else
        begin
            //؂肪܂͍Ō
            sStr5 :=sStr5 + sStr2;
        end;
        //f
        Break;
      end;
    end;
    //ݒ
    StrList3.Add(sStr5);
  end;

  //t@C܂̓Xg[͍ɕ\\
  StrStream1 := TStringStream.Create(StrList3.Text);
  Memo1.Lines.BeginUpdate;
  Memo1.Lines.LoadFromStream(StrStream1);
  Memo1.Lines.EndUpdate;
  StrListTextEx.Assign(StrList3);
  StrStream1.Free;

  //j
  StrList1.Free;
  StrList2.Free;
  StrList3.Free;

  //\XV
  StatusBar1.Panels.Items[2].Text := Format('%d', [Memo1.Lines.Count]) + 's';

  //sԍݒ
  SetLineNumber(Sender);
  if MenuShowLineNumber.Checked = TRUE then
  begin
    Memo9.Visible  := TRUE;
    Timer6.Enabled := TRUE;
  end
  else
  begin
    Memo9.Visible  := FALSE;
    Timer6.Enabled := FALSE;
  end;

  //bZ[W
  if iMaxCol1 >= CSVLIST_ITEM_MAX then
  begin
    showmessage('ڐő吔 ' +  IntToStr(CSVLIST_ITEM_MAX) +' 𒴂' + IntToStr(iMaxCol1) + ' ڂł');
  end;
end;

procedure TFormCsvText.LoadFileAuto(Sender: TObject);
begin
  if BoolFormShow = FALSE then
  begin
    Exit;
  end;

  if RadioGroup1.ItemIndex = 0 then
  begin
    //ݒ
    BoolTypeText          := TRUE;
    BoolTypeCsvAuto       := FALSE;
    BoolTypeCsvAutoFixed  := FALSE;
    BoolTypeCsvComma      := FALSE;
    BoolTypeCsvCommaFixed := FALSE;
    //Ǎ
    LoadFile(Sender);
  end
  else if RadioGroup1.ItemIndex = 1 then
  begin
    //Ǎ
    LoadTabSeparateFile(Sender);
  end
  else if RadioGroup1.ItemIndex = 2 then
  begin
    //ݒ
    BoolTypeText          := FALSE;
    BoolTypeCsvAuto       := TRUE;
    BoolTypeCsvAutoFixed  := FALSE;
    BoolTypeCsvComma      := FALSE;
    BoolTypeCsvCommaFixed := FALSE;
    //Ǎ
    LoadCsvFile(Sender);
    SetCsvFile(Sender);
  end
  else if RadioGroup1.ItemIndex = 3 then
  begin
    //ݒ
    BoolTypeText          := FALSE;
    BoolTypeCsvAuto       := FALSE;
    BoolTypeCsvAutoFixed  := TRUE;
    BoolTypeCsvComma      := FALSE;
    BoolTypeCsvCommaFixed := FALSE;
    //Ǎ
    LoadCsvFile(Sender);
    SetCsvFile(Sender);
  end
  else if RadioGroup1.ItemIndex = 4 then
  begin
    //ݒ
    BoolTypeText          := FALSE;
    BoolTypeCsvAuto       := FALSE;
    BoolTypeCsvAutoFixed  := FALSE;
    BoolTypeCsvComma      := TRUE;
    BoolTypeCsvCommaFixed := FALSE;
    //Ǎ
    LoadCsvFile(Sender);
    SetCsvFile(Sender);
  end
  else if RadioGroup1.ItemIndex = 5 then
  begin
    //ݒ
    BoolTypeText          := FALSE;
    BoolTypeCsvAuto       := FALSE;
    BoolTypeCsvAutoFixed  := FALSE;
    BoolTypeCsvComma      := FALSE;
    BoolTypeCsvCommaFixed := TRUE;
    //Ǎ
    LoadCsvFile(Sender);
    SetCsvFile(Sender);
  end
  else
  begin
    //
  end;

  //ݒ
  if MenuShowLineNumber.Checked = TRUE then
  begin
    Memo9.Visible  := TRUE;
    Timer6.Enabled := TRUE;
  end
  else
  begin
    Memo9.Visible  := FALSE;
    Timer6.Enabled := FALSE;
  end;

  //XV
  MemoStatus(Sender);
end;

procedure TFormCsvText.MemoStatus(Sender: TObject);
var
  iLine1: Integer;
  iSel1: Integer;
  iSel2: Integer;
begin
  // -1nƌ݂̃Lbgʒu̍sԍ(Ox[X)
  //@@  iLine1  := RichEdit1.Perform(EM_LINEFROMCHAR, -1, 0);
  iLine1  := Memo1.Perform(EM_LINEFROMCHAR, UINT(-1), 0);
  // -1nČ݂̃Lbgʒu̍s̐擪ʒu(Ox[X)
  //@@  iSel1   := RichEdit1.Perform(EM_LINEINDEX,    -1, 0);
  iSel1   := Memo1.Perform(EM_LINEINDEX,    UINT(-1), 0);
  //Lbgʒu(Ox[X)
  iSel2   := Memo1.SelStart;

  //\
  StatusBar1.Panels.Items[1].Text := Format('%d', [iLine1+1])          + 's,' +
                                     Format('%d', [iSel2-iSel1+1])     + '';
  StatusBar1.Panels.Items[2].Text := Format('%d', [Memo1.Lines.Count]) + 's';
end;

end.
